import { ChangeDetectorRef, Component, ContentChildren, Directive, ElementRef, EventEmitter, HostListener, Inject, Injectable, Input, LOCALE_ID, NgModule, NgZone, Output, Renderer2 } from '@angular/core';
import { CommonModule } from '@angular/common';
import { FormsModule, ReactiveFormsModule } from '@angular/forms';
import { Subject as Subject$1 } from 'rxjs/Subject';
import { Observable as Observable$1 } from 'rxjs/Observable';
import 'rxjs/add/observable/merge';
import 'rxjs/add/operator/map';
import 'rxjs/add/operator/mergeMap';
import 'rxjs/add/operator/takeUntil';
import 'rxjs/add/operator/take';
import 'rxjs/add/operator/takeLast';
import 'rxjs/add/operator/pairwise';
import 'rxjs/add/operator/share';
import 'rxjs/add/operator/distinctUntilChanged';
import 'rxjs/add/operator/filter';
import { merge as merge$2 } from 'rxjs/observable/merge';
import { interval as interval$1 } from 'rxjs/observable/interval';
import 'rxjs/add/operator/throttle';
import { addDays, addHours, addMinutes, addMonths, addSeconds, addWeeks, differenceInDays, differenceInHours, differenceInMinutes, differenceInSeconds, endOfDay, endOfHour, endOfMonth, endOfWeek, getDay, isAfter, isBefore, isDate, isSameDay, isSameMonth, isSameSecond, isWithinRange, max, setHours, setMinutes, startOfDay, startOfHour, startOfMinute, startOfMonth, startOfToday, startOfWeek, subDays, subMonths, subWeeks } from 'date-fns';

var DraggableHelper = /** @class */ (function () {
    function DraggableHelper() {
        this.currentDrag = new Subject$1();
    }
    return DraggableHelper;
}());

var MOVE_CURSOR = 'move';
var DraggableDirective = /** @class */ (function () {
    /**
     * @hidden
     */
    function DraggableDirective(element, renderer, draggableHelper, zone) {
        this.element = element;
        this.renderer = renderer;
        this.draggableHelper = draggableHelper;
        this.zone = zone;
        this.dragAxis = { x: true, y: true };
        this.dragSnapGrid = {};
        this.ghostDragEnabled = true;
        this.dragCursor = MOVE_CURSOR;
        this.dragStart = new EventEmitter();
        this.dragging = new EventEmitter();
        this.dragEnd = new EventEmitter();
        /**
         * @hidden
         */
        this.pointerDown = new Subject$1();
        /**
         * @hidden
         */
        this.pointerMove = new Subject$1();
        /**
         * @hidden
         */
        this.pointerUp = new Subject$1();
        this.eventListenerSubscriptions = {};
    }
    DraggableDirective.prototype.ngOnInit = function () {
        var _this = this;
        this.checkEventListeners();
        var pointerDrag = this.pointerDown
            .filter(function () { return _this.canDrag(); })
            .flatMap(function (pointerDownEvent) {
            pointerDownEvent.event.preventDefault();
            _this.zone.run(function () {
                _this.dragStart.next({ x: 0, y: 0 });
            });
            _this.setCursor(_this.dragCursor);
            var currentDrag = new Subject$1();
            _this.draggableHelper.currentDrag.next(currentDrag);
            var pointerMove = _this.pointerMove
                .map(function (pointerMoveEvent) {
                pointerMoveEvent.event.preventDefault();
                return {
                    currentDrag: currentDrag,
                    x: pointerMoveEvent.clientX - pointerDownEvent.clientX,
                    y: pointerMoveEvent.clientY - pointerDownEvent.clientY,
                    clientX: pointerMoveEvent.clientX,
                    clientY: pointerMoveEvent.clientY
                };
            })
                .map(function (moveData) {
                if (_this.dragSnapGrid.x) {
                    moveData.x =
                        Math.floor(moveData.x / _this.dragSnapGrid.x) *
                            _this.dragSnapGrid.x;
                }
                if (_this.dragSnapGrid.y) {
                    moveData.y =
                        Math.floor(moveData.y / _this.dragSnapGrid.y) *
                            _this.dragSnapGrid.y;
                }
                return moveData;
            })
                .map(function (moveData) {
                if (!_this.dragAxis.x) {
                    moveData.x = 0;
                }
                if (!_this.dragAxis.y) {
                    moveData.y = 0;
                }
                return moveData;
            })
                .filter(function (_a) {
                var x = _a.x, y = _a.y;
                return !_this.validateDrag || _this.validateDrag({ x: x, y: y });
            })
                .takeUntil(Observable$1.merge(_this.pointerUp, _this.pointerDown));
            pointerMove.takeLast(1).subscribe(function (_a) {
                var x = _a.x, y = _a.y;
                _this.zone.run(function () {
                    _this.dragEnd.next({ x: x, y: y });
                });
                currentDrag.complete();
                _this.setCssTransform(null);
                if (_this.ghostDragEnabled) {
                    _this.renderer.setStyle(_this.element.nativeElement, 'pointerEvents', null);
                }
            });
            _this.pointerMove.next(pointerDownEvent);
            return pointerMove;
        })
            .share();
        Observable$1.merge(pointerDrag.take(1).map(function (value) { return [, value]; }), pointerDrag.pairwise())
            .filter(function (_a) {
            var previous = _a[0], next = _a[1];
            if (!previous) {
                return true;
            }
            return previous.x !== next.x || previous.y !== next.y;
        })
            .map(function (_a) {
            var previous = _a[0], next = _a[1];
            return next;
        })
            .subscribe(function (_a) {
            var x = _a.x, y = _a.y, currentDrag = _a.currentDrag, clientX = _a.clientX, clientY = _a.clientY;
            _this.zone.run(function () {
                _this.dragging.next({ x: x, y: y });
            });
            if (_this.ghostDragEnabled) {
                _this.renderer.setStyle(_this.element.nativeElement, 'pointerEvents', 'none');
            }
            _this.setCssTransform("translate(" + x + "px, " + y + "px)");
            currentDrag.next({
                clientX: clientX,
                clientY: clientY,
                dropData: _this.dropData
            });
        });
    };
    DraggableDirective.prototype.ngOnChanges = function (changes) {
        if (changes['dragAxis']) {
            this.checkEventListeners();
        }
    };
    DraggableDirective.prototype.ngOnDestroy = function () {
        this.unsubscribeEventListeners();
        this.pointerDown.complete();
        this.pointerMove.complete();
        this.pointerUp.complete();
    };
    DraggableDirective.prototype.checkEventListeners = function () {
        var _this = this;
        var canDrag = this.canDrag();
        var hasEventListeners = Object.keys(this.eventListenerSubscriptions).length > 0;
        if (canDrag && !hasEventListeners) {
            this.zone.runOutsideAngular(function () {
                _this.eventListenerSubscriptions.mousedown = _this.renderer.listen(_this.element.nativeElement, 'mousedown', function (event) {
                    _this.onMouseDown(event);
                });
                _this.eventListenerSubscriptions.mouseup = _this.renderer.listen('document', 'mouseup', function (event) {
                    _this.onMouseUp(event);
                });
                _this.eventListenerSubscriptions.touchstart = _this.renderer.listen(_this.element.nativeElement, 'touchstart', function (event) {
                    _this.onTouchStart(event);
                });
                _this.eventListenerSubscriptions.touchend = _this.renderer.listen('document', 'touchend', function (event) {
                    _this.onTouchEnd(event);
                });
                _this.eventListenerSubscriptions.touchcancel = _this.renderer.listen('document', 'touchcancel', function (event) {
                    _this.onTouchEnd(event);
                });
                _this.eventListenerSubscriptions.mouseenter = _this.renderer.listen(_this.element.nativeElement, 'mouseenter', function () {
                    _this.onMouseEnter();
                });
                _this.eventListenerSubscriptions.mouseleave = _this.renderer.listen(_this.element.nativeElement, 'mouseleave', function () {
                    _this.onMouseLeave();
                });
            });
        }
        else if (!canDrag && hasEventListeners) {
            this.unsubscribeEventListeners();
        }
    };
    DraggableDirective.prototype.onMouseDown = function (event) {
        var _this = this;
        if (!this.eventListenerSubscriptions.mousemove) {
            this.eventListenerSubscriptions.mousemove = this.renderer.listen('document', 'mousemove', function (mouseMoveEvent) {
                _this.pointerMove.next({
                    event: mouseMoveEvent,
                    clientX: mouseMoveEvent.clientX,
                    clientY: mouseMoveEvent.clientY
                });
            });
        }
        this.pointerDown.next({
            event: event,
            clientX: event.clientX,
            clientY: event.clientY
        });
    };
    DraggableDirective.prototype.onMouseUp = function (event) {
        if (this.eventListenerSubscriptions.mousemove) {
            this.eventListenerSubscriptions.mousemove();
            delete this.eventListenerSubscriptions.mousemove;
        }
        this.pointerUp.next({
            event: event,
            clientX: event.clientX,
            clientY: event.clientY
        });
    };
    DraggableDirective.prototype.onTouchStart = function (event) {
        var _this = this;
        if (!this.eventListenerSubscriptions.touchmove) {
            this.eventListenerSubscriptions.touchmove = this.renderer.listen('document', 'touchmove', function (touchMoveEvent) {
                _this.pointerMove.next({
                    event: touchMoveEvent,
                    clientX: touchMoveEvent.targetTouches[0].clientX,
                    clientY: touchMoveEvent.targetTouches[0].clientY
                });
            });
        }
        this.pointerDown.next({
            event: event,
            clientX: event.touches[0].clientX,
            clientY: event.touches[0].clientY
        });
    };
    DraggableDirective.prototype.onTouchEnd = function (event) {
        if (this.eventListenerSubscriptions.touchmove) {
            this.eventListenerSubscriptions.touchmove();
            delete this.eventListenerSubscriptions.touchmove;
        }
        this.pointerUp.next({
            event: event,
            clientX: event.changedTouches[0].clientX,
            clientY: event.changedTouches[0].clientY
        });
    };
    DraggableDirective.prototype.onMouseEnter = function () {
        this.setCursor(this.dragCursor);
    };
    DraggableDirective.prototype.onMouseLeave = function () {
        this.setCursor(null);
    };
    DraggableDirective.prototype.setCssTransform = function (value) {
        if (this.ghostDragEnabled) {
            this.renderer.setStyle(this.element.nativeElement, 'transform', value);
            this.renderer.setStyle(this.element.nativeElement, '-webkit-transform', value);
            this.renderer.setStyle(this.element.nativeElement, '-ms-transform', value);
            this.renderer.setStyle(this.element.nativeElement, '-moz-transform', value);
            this.renderer.setStyle(this.element.nativeElement, '-o-transform', value);
        }
    };
    DraggableDirective.prototype.canDrag = function () {
        return this.dragAxis.x || this.dragAxis.y;
    };
    DraggableDirective.prototype.setCursor = function (value) {
        this.renderer.setStyle(this.element.nativeElement, 'cursor', value);
    };
    DraggableDirective.prototype.unsubscribeEventListeners = function () {
        var _this = this;
        Object.keys(this.eventListenerSubscriptions).forEach(function (type) {
            _this.eventListenerSubscriptions[type]();
            delete _this.eventListenerSubscriptions[type];
        });
    };
    DraggableDirective.decorators = [
        { type: Directive, args: [{
                    selector: '[mwlDraggable]'
                },] },
    ];
    /** @nocollapse */
    DraggableDirective.ctorParameters = function () { return [
        { type: ElementRef, },
        { type: Renderer2, },
        { type: DraggableHelper, },
        { type: NgZone, },
    ]; };
    DraggableDirective.propDecorators = {
        'dropData': [{ type: Input },],
        'dragAxis': [{ type: Input },],
        'dragSnapGrid': [{ type: Input },],
        'ghostDragEnabled': [{ type: Input },],
        'validateDrag': [{ type: Input },],
        'dragCursor': [{ type: Input },],
        'dragStart': [{ type: Output },],
        'dragging': [{ type: Output },],
        'dragEnd': [{ type: Output },],
    };
    return DraggableDirective;
}());

function isCoordinateWithinRectangle(clientX, clientY, rect) {
    return (clientX >= rect.left &&
        clientX <= rect.right &&
        clientY >= rect.top &&
        clientY <= rect.bottom);
}
var DroppableDirective = /** @class */ (function () {
    function DroppableDirective(element, draggableHelper, zone) {
        this.element = element;
        this.draggableHelper = draggableHelper;
        this.zone = zone;
        this.dragEnter = new EventEmitter();
        this.dragLeave = new EventEmitter();
        this.dragOver = new EventEmitter();
        this.drop = new EventEmitter();
    }
    DroppableDirective.prototype.ngOnInit = function () {
        var _this = this;
        this.currentDragSubscription = this.draggableHelper.currentDrag.subscribe(function (drag) {
            var droppableRectangle = _this.element.nativeElement.getBoundingClientRect();
            var currentDragDropData;
            var overlaps = drag.map(function (_a) {
                var clientX = _a.clientX, clientY = _a.clientY, dropData = _a.dropData;
                currentDragDropData = dropData;
                return isCoordinateWithinRectangle(clientX, clientY, droppableRectangle);
            });
            var overlapsChanged = overlaps.distinctUntilChanged();
            var dragOverActive; // TODO - see if there's a way of doing this via rxjs
            overlapsChanged.filter(function (overlapsNow) { return overlapsNow; }).subscribe(function () {
                dragOverActive = true;
                _this.zone.run(function () {
                    _this.dragEnter.next({
                        dropData: currentDragDropData
                    });
                });
            });
            overlaps.filter(function (overlapsNow) { return overlapsNow; }).subscribe(function () {
                _this.zone.run(function () {
                    _this.dragOver.next({
                        dropData: currentDragDropData
                    });
                });
            });
            overlapsChanged
                .pairwise()
                .filter(function (_a) {
                var didOverlap = _a[0], overlapsNow = _a[1];
                return didOverlap && !overlapsNow;
            })
                .subscribe(function () {
                dragOverActive = false;
                _this.zone.run(function () {
                    _this.dragLeave.next({
                        dropData: currentDragDropData
                    });
                });
            });
            drag.flatMap(function () { return overlaps; }).subscribe({
                complete: function () {
                    if (dragOverActive) {
                        _this.zone.run(function () {
                            _this.drop.next({
                                dropData: currentDragDropData
                            });
                        });
                    }
                }
            });
        });
    };
    DroppableDirective.prototype.ngOnDestroy = function () {
        this.currentDragSubscription.unsubscribe();
    };
    DroppableDirective.decorators = [
        { type: Directive, args: [{
                    selector: '[mwlDroppable]'
                },] },
    ];
    /** @nocollapse */
    DroppableDirective.ctorParameters = function () { return [
        { type: ElementRef, },
        { type: DraggableHelper, },
        { type: NgZone, },
    ]; };
    DroppableDirective.propDecorators = {
        'dragEnter': [{ type: Output },],
        'dragLeave': [{ type: Output },],
        'dragOver': [{ type: Output },],
        'drop': [{ type: Output },],
    };
    return DroppableDirective;
}());

var DragAndDropModule = /** @class */ (function () {
    function DragAndDropModule() {
    }
    DragAndDropModule.forRoot = function () {
        return {
            ngModule: DragAndDropModule,
            providers: [DraggableHelper]
        };
    };
    DragAndDropModule.decorators = [
        { type: NgModule, args: [{
                    declarations: [DraggableDirective, DroppableDirective],
                    exports: [DraggableDirective, DroppableDirective]
                },] },
    ];
    /** @nocollapse */
    DragAndDropModule.ctorParameters = function () { return []; };
    return DragAndDropModule;
}());

/**
 * An element placed inside a `mwlResizable` directive to be used as a drag and resize handle
 *
 * For example
 *
 * ```
 * &lt;div mwlResizable&gt;
 *   &lt;div mwlResizeHandle [resizeEdges]="{bottom: true, right: true}"&gt;&lt;/div&gt;
 * &lt;/div&gt;
 * ```
 */
var ResizeHandle = (function () {
    function ResizeHandle(renderer, element, zone) {
        this.renderer = renderer;
        this.element = element;
        this.zone = zone;
        /**
         * The `Edges` object that contains the edges of the parent element that dragging the handle will trigger a resize on
         */
        this.resizeEdges = {};
        this.eventListeners = {};
    }
    ResizeHandle.prototype.ngOnDestroy = function () {
        this.unsubscribeEventListeners();
    };
    /**
     * @private
     */
    ResizeHandle.prototype.onMousedown = function (event, mouseX, mouseY) {
        var _this = this;
        event.preventDefault();
        this.zone.runOutsideAngular(function () {
            if (!_this.eventListeners.touchmove) {
                _this.eventListeners.touchmove = _this.renderer.listen(_this.element.nativeElement, 'touchmove', function (event) {
                    _this.onMousemove(event, event.targetTouches[0].clientX, event.targetTouches[0].clientY);
                });
            }
            if (!_this.eventListeners.mousemove) {
                _this.eventListeners.mousemove = _this.renderer.listen(_this.element.nativeElement, 'mousemove', function (event) {
                    _this.onMousemove(event, event.clientX, event.clientY);
                });
            }
            _this.resizable.mousedown.next({ mouseX: mouseX, mouseY: mouseY, edges: _this.resizeEdges });
        });
    };
    /**
     * @private
     */
    ResizeHandle.prototype.onMouseup = function (mouseX, mouseY) {
        var _this = this;
        this.zone.runOutsideAngular(function () {
            _this.unsubscribeEventListeners();
            _this.resizable.mouseup.next({ mouseX: mouseX, mouseY: mouseY, edges: _this.resizeEdges });
        });
    };
    ResizeHandle.prototype.onMousemove = function (event, mouseX, mouseY) {
        this.resizable.mousemove.next({ mouseX: mouseX, mouseY: mouseY, edges: this.resizeEdges, event: event });
    };
    ResizeHandle.prototype.unsubscribeEventListeners = function () {
        var _this = this;
        Object.keys(this.eventListeners).forEach(function (type) {
            _this.eventListeners[type]();
            delete _this.eventListeners[type];
        });
    };
    return ResizeHandle;
}());
ResizeHandle.decorators = [
    { type: Directive, args: [{
                selector: '[mwlResizeHandle]'
            },] },
];
/** @nocollapse */
ResizeHandle.ctorParameters = function () { return [
    { type: Renderer2, },
    { type: ElementRef, },
    { type: NgZone, },
]; };
ResizeHandle.propDecorators = {
    'resizeEdges': [{ type: Input },],
    'onMousedown': [{ type: HostListener, args: ['touchstart', ['$event', '$event.touches[0].clientX', '$event.touches[0].clientY'],] }, { type: HostListener, args: ['mousedown', ['$event', '$event.clientX', '$event.clientY'],] },],
    'onMouseup': [{ type: HostListener, args: ['touchend', ['$event.changedTouches[0].clientX', '$event.changedTouches[0].clientY'],] }, { type: HostListener, args: ['touchcancel', ['$event.changedTouches[0].clientX', '$event.changedTouches[0].clientY'],] }, { type: HostListener, args: ['mouseup', ['$event.clientX', '$event.clientY'],] },],
};

function isNumberCloseTo(value1, value2, precision) {
    if (precision === void 0) { precision = 3; }
    var diff = Math.abs(value1 - value2);
    return diff < precision;
}
function getNewBoundingRectangle(startingRect, edges, clientX, clientY) {
    var newBoundingRect = {
        top: startingRect.top,
        bottom: startingRect.bottom,
        left: startingRect.left,
        right: startingRect.right
    };
    if (edges.top) {
        newBoundingRect.top += clientY;
    }
    if (edges.bottom) {
        newBoundingRect.bottom += clientY;
    }
    if (edges.left) {
        newBoundingRect.left += clientX;
    }
    if (edges.right) {
        newBoundingRect.right += clientX;
    }
    newBoundingRect.height = newBoundingRect.bottom - newBoundingRect.top;
    newBoundingRect.width = newBoundingRect.right - newBoundingRect.left;
    return newBoundingRect;
}
function getElementRect(element, ghostElementPositioning) {
    if (ghostElementPositioning === 'absolute') {
        return {
            height: element.nativeElement.offsetHeight,
            width: element.nativeElement.offsetWidth,
            top: element.nativeElement.offsetTop,
            bottom: element.nativeElement.offsetHeight + element.nativeElement.offsetTop,
            left: element.nativeElement.offsetLeft,
            right: element.nativeElement.offsetWidth + element.nativeElement.offsetLeft
        };
    }
    else {
        var boundingRect = element.nativeElement.getBoundingClientRect();
        return {
            height: boundingRect.height,
            width: boundingRect.width,
            top: boundingRect.top,
            bottom: boundingRect.bottom,
            left: boundingRect.left,
            right: boundingRect.right,
            scrollTop: element.nativeElement.scrollTop,
            scrollLeft: element.nativeElement.scrollLeft
        };
    }
}
function isWithinBoundingY(_a) {
    var clientY = _a.clientY, rect = _a.rect;
    return clientY >= rect.top && clientY <= rect.bottom;
}
function isWithinBoundingX(_a) {
    var clientX = _a.clientX, rect = _a.rect;
    return clientX >= rect.left && clientX <= rect.right;
}
function getResizeEdges(_a) {
    var clientX = _a.clientX, clientY = _a.clientY, elm = _a.elm, allowedEdges = _a.allowedEdges, cursorPrecision = _a.cursorPrecision;
    var elmPosition = elm.nativeElement.getBoundingClientRect();
    var edges = {};
    if (allowedEdges.left &&
        isNumberCloseTo(clientX, elmPosition.left, cursorPrecision) &&
        isWithinBoundingY({ clientY: clientY, rect: elmPosition })) {
        edges.left = true;
    }
    if (allowedEdges.right &&
        isNumberCloseTo(clientX, elmPosition.right, cursorPrecision) &&
        isWithinBoundingY({ clientY: clientY, rect: elmPosition })) {
        edges.right = true;
    }
    if (allowedEdges.top &&
        isNumberCloseTo(clientY, elmPosition.top, cursorPrecision) &&
        isWithinBoundingX({ clientX: clientX, rect: elmPosition })) {
        edges.top = true;
    }
    if (allowedEdges.bottom &&
        isNumberCloseTo(clientY, elmPosition.bottom, cursorPrecision) &&
        isWithinBoundingX({ clientX: clientX, rect: elmPosition })) {
        edges.bottom = true;
    }
    return edges;
}
var DEFAULT_RESIZE_CURSORS = Object.freeze({
    topLeft: 'nw-resize',
    topRight: 'ne-resize',
    bottomLeft: 'sw-resize',
    bottomRight: 'se-resize',
    leftOrRight: 'ew-resize',
    topOrBottom: 'ns-resize'
});
function getResizeCursor(edges, cursors) {
    if (edges.left && edges.top) {
        return cursors.topLeft;
    }
    else if (edges.right && edges.top) {
        return cursors.topRight;
    }
    else if (edges.left && edges.bottom) {
        return cursors.bottomLeft;
    }
    else if (edges.right && edges.bottom) {
        return cursors.bottomRight;
    }
    else if (edges.left || edges.right) {
        return cursors.leftOrRight;
    }
    else if (edges.top || edges.bottom) {
        return cursors.topOrBottom;
    }
    else {
        return '';
    }
}
function getEdgesDiff(_a) {
    var edges = _a.edges, initialRectangle = _a.initialRectangle, newRectangle = _a.newRectangle;
    var edgesDiff = {};
    Object.keys(edges).forEach(function (edge) {
        edgesDiff[edge] = newRectangle[edge] - initialRectangle[edge];
    });
    return edgesDiff;
}
var RESIZE_ACTIVE_CLASS = 'resize-active';
var RESIZE_LEFT_HOVER_CLASS = 'resize-left-hover';
var RESIZE_RIGHT_HOVER_CLASS = 'resize-right-hover';
var RESIZE_TOP_HOVER_CLASS = 'resize-top-hover';
var RESIZE_BOTTOM_HOVER_CLASS = 'resize-bottom-hover';
var RESIZE_GHOST_ELEMENT_CLASS = 'resize-ghost-element';
var MOUSE_MOVE_THROTTLE_MS = 50;
/**
 * Place this on an element to make it resizable
 *
 * For example
 *
 * ```
 * &lt;div mwlResizable [resizeEdges]="{bottom: true, right: true, top: true, left: true}" [enableGhostResize]="true"&gt;&lt;/div&gt;
 * ```
 */
var Resizable = (function () {
    /**
     * @hidden
     */
    function Resizable(renderer, elm, zone) {
        this.renderer = renderer;
        this.elm = elm;
        this.zone = zone;
        /**
         * The edges that an element can be resized from. Pass an object like `{top: true, bottom: false}`. By default no edges can be resized.
         */
        this.resizeEdges = {};
        /**
         * Set to `true` to enable a temporary resizing effect of the element in between the `resizeStart` and `resizeEnd` events.
         */
        this.enableGhostResize = false;
        /**
         * A snap grid that resize events will be locked to.
         *
         * e.g. to only allow the element to be resized every 10px set it to `{left: 10, right: 10}`
         */
        this.resizeSnapGrid = {};
        /**
         * The mouse cursors that will be set on the resize edges
         */
        this.resizeCursors = DEFAULT_RESIZE_CURSORS;
        /**
         * Mouse over thickness to active cursor.
         */
        this.resizeCursorPrecision = 3;
        /**
         * Define the positioning of the ghost element (can be fixed or absolute)
         */
        this.ghostElementPositioning = 'fixed';
        /**
         * Called when the mouse is pressed and a resize event is about to begin. `$event` is a `ResizeEvent` object.
         */
        this.resizeStart = new EventEmitter();
        /**
         * Called as the mouse is dragged after a resize event has begun. `$event` is a `ResizeEvent` object.
         */
        this.resizing = new EventEmitter();
        /**
         * Called after the mouse is released after a resize event. `$event` is a `ResizeEvent` object.
         */
        this.resizeEnd = new EventEmitter();
        /**
         * @hidden
         */
        this.mouseup = new Subject$1();
        /**
         * @hidden
         */
        this.mousedown = new Subject$1();
        /**
         * @hidden
         */
        this.mousemove = new Subject$1();
        this.pointerEventListenerSubscriptions = {};
        this.pointerEventListeners = PointerEventListeners.getInstance(renderer, zone);
    }
    /**
     * @hidden
     */
    Resizable.prototype.ngOnInit = function () {
        var _this = this;
        // TODO - use some fancy Observable.merge's for this
        this.pointerEventListenerSubscriptions.pointerDown = this.pointerEventListeners.pointerDown.subscribe(function (_a) {
            var clientX = _a.clientX, clientY = _a.clientY;
            _this.mousedown.next({ clientX: clientX, clientY: clientY });
        });
        this.pointerEventListenerSubscriptions.pointerMove = this.pointerEventListeners.pointerMove.subscribe(function (_a) {
            var clientX = _a.clientX, clientY = _a.clientY, event = _a.event;
            _this.mousemove.next({ clientX: clientX, clientY: clientY, event: event });
        });
        this.pointerEventListenerSubscriptions.pointerUp = this.pointerEventListeners.pointerUp.subscribe(function (_a) {
            var clientX = _a.clientX, clientY = _a.clientY;
            _this.mouseup.next({ clientX: clientX, clientY: clientY });
        });
        var currentResize;
        var removeGhostElement = function () {
            if (currentResize.clonedNode) {
                _this.elm.nativeElement.parentElement.removeChild(currentResize.clonedNode);
                _this.renderer.setStyle(_this.elm.nativeElement, 'visibility', 'inherit');
            }
        };
        var mouseMove = this.mousemove.share();
        mouseMove
            .filter(function () { return !!currentResize; })
            .subscribe(function (_a) {
            var event = _a.event;
            event.preventDefault();
        });
        mouseMove.throttle(function (val) { return interval$1(MOUSE_MOVE_THROTTLE_MS); }).subscribe(function (_a) {
            var clientX = _a.clientX, clientY = _a.clientY;
            var resizeEdges = getResizeEdges({
                clientX: clientX, clientY: clientY,
                elm: _this.elm,
                allowedEdges: _this.resizeEdges,
                cursorPrecision: _this.resizeCursorPrecision
            });
            var resizeCursors = Object.assign({}, DEFAULT_RESIZE_CURSORS, _this.resizeCursors);
            var cursor = currentResize ? '' : getResizeCursor(resizeEdges, resizeCursors);
            _this.renderer.setStyle(_this.elm.nativeElement, 'cursor', cursor);
            _this.setElementClass(_this.elm, RESIZE_ACTIVE_CLASS, !!currentResize);
            _this.setElementClass(_this.elm, RESIZE_LEFT_HOVER_CLASS, resizeEdges.left === true);
            _this.setElementClass(_this.elm, RESIZE_RIGHT_HOVER_CLASS, resizeEdges.right === true);
            _this.setElementClass(_this.elm, RESIZE_TOP_HOVER_CLASS, resizeEdges.top === true);
            _this.setElementClass(_this.elm, RESIZE_BOTTOM_HOVER_CLASS, resizeEdges.bottom === true);
        });
        var mousedrag = this.mousedown.flatMap(function (startCoords) {
            var getDiff = function (moveCoords) {
                return {
                    clientX: moveCoords.clientX - startCoords.clientX,
                    clientY: moveCoords.clientY - startCoords.clientY
                };
            };
            var getSnapGrid = function () {
                var snapGrid = { x: 1, y: 1 };
                if (currentResize) {
                    if (_this.resizeSnapGrid.left && currentResize.edges.left) {
                        snapGrid.x = +_this.resizeSnapGrid.left;
                    }
                    else if (_this.resizeSnapGrid.right && currentResize.edges.right) {
                        snapGrid.x = +_this.resizeSnapGrid.right;
                    }
                    if (_this.resizeSnapGrid.top && currentResize.edges.top) {
                        snapGrid.y = +_this.resizeSnapGrid.top;
                    }
                    else if (_this.resizeSnapGrid.bottom && currentResize.edges.bottom) {
                        snapGrid.y = +_this.resizeSnapGrid.bottom;
                    }
                }
                return snapGrid;
            };
            var getGrid = function (coords, snapGrid) {
                return {
                    x: Math.ceil(coords.clientX / snapGrid.x),
                    y: Math.ceil(coords.clientY / snapGrid.y)
                };
            };
            return merge$2(mouseMove.take(1).map(function (coords) { return [, coords]; }), mouseMove.pairwise()).map(function (_a) {
                var previousCoords = _a[0], newCoords = _a[1];
                return [previousCoords ? getDiff(previousCoords) : previousCoords, getDiff(newCoords)];
            }).filter(function (_a) {
                var previousCoords = _a[0], newCoords = _a[1];
                if (!previousCoords) {
                    return true;
                }
                var snapGrid = getSnapGrid();
                var previousGrid = getGrid(previousCoords, snapGrid);
                var newGrid = getGrid(newCoords, snapGrid);
                return (previousGrid.x !== newGrid.x || previousGrid.y !== newGrid.y);
            }).map(function (_a) {
                var newCoords = _a[1];
                var snapGrid = getSnapGrid();
                return {
                    clientX: Math.round(newCoords.clientX / snapGrid.x) * snapGrid.x,
                    clientY: Math.round(newCoords.clientY / snapGrid.y) * snapGrid.y
                };
            }).takeUntil(merge$2(_this.mouseup, _this.mousedown));
        }).filter(function () { return !!currentResize; });
        mousedrag.map(function (_a) {
            var clientX = _a.clientX, clientY = _a.clientY;
            return getNewBoundingRectangle(currentResize.startingRect, currentResize.edges, clientX, clientY);
        }).filter(function (newBoundingRect) {
            return newBoundingRect.height > 0 && newBoundingRect.width > 0;
        }).filter(function (newBoundingRect) {
            return _this.validateResize ? _this.validateResize({
                rectangle: newBoundingRect,
                edges: getEdgesDiff({
                    edges: currentResize.edges,
                    initialRectangle: currentResize.startingRect,
                    newRectangle: newBoundingRect
                })
            }) : true;
        }).subscribe(function (newBoundingRect) {
            if (currentResize.clonedNode) {
                _this.renderer.setStyle(currentResize.clonedNode, 'height', newBoundingRect.height + "px");
                _this.renderer.setStyle(currentResize.clonedNode, 'width', newBoundingRect.width + "px");
                _this.renderer.setStyle(currentResize.clonedNode, 'top', newBoundingRect.top + "px");
                _this.renderer.setStyle(currentResize.clonedNode, 'left', newBoundingRect.left + "px");
            }
            _this.zone.run(function () {
                _this.resizing.emit({
                    edges: getEdgesDiff({
                        edges: currentResize.edges,
                        initialRectangle: currentResize.startingRect,
                        newRectangle: newBoundingRect
                    }),
                    rectangle: newBoundingRect
                });
            });
            currentResize.currentRect = newBoundingRect;
        });
        this.mousedown.map(function (_a) {
            var clientX = _a.clientX, clientY = _a.clientY, edges = _a.edges;
            return edges || getResizeEdges({
                clientX: clientX, clientY: clientY,
                elm: _this.elm,
                allowedEdges: _this.resizeEdges,
                cursorPrecision: _this.resizeCursorPrecision
            });
        }).filter(function (edges) {
            return Object.keys(edges).length > 0;
        }).subscribe(function (edges) {
            if (currentResize) {
                removeGhostElement();
            }
            var startingRect = getElementRect(_this.elm, _this.ghostElementPositioning);
            currentResize = {
                edges: edges,
                startingRect: startingRect,
                currentRect: startingRect
            };
            if (_this.enableGhostResize) {
                currentResize.clonedNode = _this.elm.nativeElement.cloneNode(true);
                var resizeCursors = Object.assign({}, DEFAULT_RESIZE_CURSORS, _this.resizeCursors);
                _this.elm.nativeElement.parentElement.appendChild(currentResize.clonedNode);
                _this.renderer.setStyle(_this.elm.nativeElement, 'visibility', 'hidden');
                _this.renderer.setStyle(currentResize.clonedNode, 'position', _this.ghostElementPositioning);
                _this.renderer.setStyle(currentResize.clonedNode, 'left', currentResize.startingRect.left + "px");
                _this.renderer.setStyle(currentResize.clonedNode, 'top', currentResize.startingRect.top + "px");
                _this.renderer.setStyle(currentResize.clonedNode, 'height', currentResize.startingRect.height + "px");
                _this.renderer.setStyle(currentResize.clonedNode, 'width', currentResize.startingRect.width + "px");
                _this.renderer.setStyle(currentResize.clonedNode, 'cursor', getResizeCursor(currentResize.edges, resizeCursors));
                _this.renderer.addClass(currentResize.clonedNode, RESIZE_GHOST_ELEMENT_CLASS);
                currentResize.clonedNode.scrollTop = currentResize.startingRect.scrollTop;
                currentResize.clonedNode.scrollLeft = currentResize.startingRect.scrollLeft;
            }
            _this.zone.run(function () {
                _this.resizeStart.emit({
                    edges: getEdgesDiff({ edges: edges, initialRectangle: startingRect, newRectangle: startingRect }),
                    rectangle: getNewBoundingRectangle(startingRect, {}, 0, 0)
                });
            });
        });
        this.mouseup.subscribe(function () {
            if (currentResize) {
                _this.renderer.removeClass(_this.elm.nativeElement, RESIZE_ACTIVE_CLASS);
                _this.zone.run(function () {
                    _this.resizeEnd.emit({
                        edges: getEdgesDiff({
                            edges: currentResize.edges,
                            initialRectangle: currentResize.startingRect,
                            newRectangle: currentResize.currentRect
                        }),
                        rectangle: currentResize.currentRect
                    });
                });
                removeGhostElement();
                currentResize = null;
            }
        });
    };
    /**
     * @hidden
     */
    Resizable.prototype.ngAfterViewInit = function () {
        var _this = this;
        this.resizeHandles.forEach(function (handle) {
            handle.resizable = _this;
        });
    };
    /**
     * @hidden
     */
    Resizable.prototype.ngOnDestroy = function () {
        this.mousedown.complete();
        this.mouseup.complete();
        this.mousemove.complete();
        this.pointerEventListenerSubscriptions.pointerDown.unsubscribe();
        this.pointerEventListenerSubscriptions.pointerMove.unsubscribe();
        this.pointerEventListenerSubscriptions.pointerUp.unsubscribe();
    };
    Resizable.prototype.setElementClass = function (elm, name, add) {
        if (add) {
            this.renderer.addClass(elm.nativeElement, name);
        }
        else {
            this.renderer.removeClass(elm.nativeElement, name);
        }
    };
    return Resizable;
}());
Resizable.decorators = [
    { type: Directive, args: [{
                selector: '[mwlResizable]'
            },] },
];
/** @nocollapse */
Resizable.ctorParameters = function () { return [
    { type: Renderer2, },
    { type: ElementRef, },
    { type: NgZone, },
]; };
Resizable.propDecorators = {
    'validateResize': [{ type: Input },],
    'resizeEdges': [{ type: Input },],
    'enableGhostResize': [{ type: Input },],
    'resizeSnapGrid': [{ type: Input },],
    'resizeCursors': [{ type: Input },],
    'resizeCursorPrecision': [{ type: Input },],
    'ghostElementPositioning': [{ type: Input },],
    'resizeStart': [{ type: Output },],
    'resizing': [{ type: Output },],
    'resizeEnd': [{ type: Output },],
    'resizeHandles': [{ type: ContentChildren, args: [ResizeHandle,] },],
};
var PointerEventListeners = (function () {
    function PointerEventListeners(renderer, zone) {
        this.pointerDown = new Observable$1(function (observer) {
            var unsubscribeMouseDown, unsubscribeTouchStart;
            zone.runOutsideAngular(function () {
                unsubscribeMouseDown = renderer.listen('document', 'mousedown', function (event) {
                    observer.next({ clientX: event.clientX, clientY: event.clientY, event: event });
                });
                unsubscribeTouchStart = renderer.listen('document', 'touchstart', function (event) {
                    observer.next({ clientX: event.touches[0].clientX, clientY: event.touches[0].clientY, event: event });
                });
            });
            return function () {
                unsubscribeMouseDown();
                unsubscribeTouchStart();
            };
        }).share();
        this.pointerMove = new Observable$1(function (observer) {
            var unsubscribeMouseMove, unsubscribeTouchMove;
            zone.runOutsideAngular(function () {
                unsubscribeMouseMove = renderer.listen('document', 'mousemove', function (event) {
                    observer.next({ clientX: event.clientX, clientY: event.clientY, event: event });
                });
                unsubscribeTouchMove = renderer.listen('document', 'touchmove', function (event) {
                    observer.next({ clientX: event.targetTouches[0].clientX, clientY: event.targetTouches[0].clientY, event: event });
                });
            });
            return function () {
                unsubscribeMouseMove();
                unsubscribeTouchMove();
            };
        }).share();
        this.pointerUp = new Observable$1(function (observer) {
            var unsubscribeMouseUp, unsubscribeTouchEnd, unsubscribeTouchCancel;
            zone.runOutsideAngular(function () {
                unsubscribeMouseUp = renderer.listen('document', 'mouseup', function (event) {
                    observer.next({ clientX: event.clientX, clientY: event.clientY, event: event });
                });
                unsubscribeTouchEnd = renderer.listen('document', 'touchend', function (event) {
                    observer.next({ clientX: event.changedTouches[0].clientX, clientY: event.changedTouches[0].clientY, event: event });
                });
                unsubscribeTouchCancel = renderer.listen('document', 'touchcancel', function (event) {
                    observer.next({ clientX: event.changedTouches[0].clientX, clientY: event.changedTouches[0].clientY, event: event });
                });
            });
            return function () {
                unsubscribeMouseUp();
                unsubscribeTouchEnd();
                unsubscribeTouchCancel();
            };
        }).share();
    }
    PointerEventListeners.getInstance = function (renderer, zone) {
        if (!PointerEventListeners.instance) {
            PointerEventListeners.instance = new PointerEventListeners(renderer, zone);
        }
        return PointerEventListeners.instance;
    };
    return PointerEventListeners;
}());

var ResizableModule = (function () {
    function ResizableModule() {
    }
    return ResizableModule;
}());
ResizableModule.decorators = [
    { type: NgModule, args: [{
                declarations: [Resizable, ResizeHandle],
                exports: [Resizable, ResizeHandle]
            },] },
];
/** @nocollapse */
ResizableModule.ctorParameters = function () { return []; };

// tslint:disable-line

/**
 * @param {?} __0
 * @return {?}
 */
function getEventsInPeriod({ events, periodStart, periodEnd }) {
    return events.filter((event) => isEventIsPeriod({ event, periodStart, periodEnd }));
}
/**
 * @param {?} __0
 * @return {?}
 */
function isEventIsPeriod({ event, periodStart, periodEnd }) {
    const /** @type {?} */ eventStart = event.start;
    const /** @type {?} */ eventEnd = event.end || event.start;
    if (eventStart > periodStart && eventStart < periodEnd) {
        return true;
    }
    if (eventEnd > periodStart && eventEnd < periodEnd) {
        return true;
    }
    if (eventStart < periodStart && eventEnd > periodEnd) {
        return true;
    }
    if (isSameSecond(eventStart, periodStart) ||
        isSameSecond(eventStart, periodEnd)) {
        return true;
    }
    if (isSameSecond(eventEnd, periodStart) ||
        isSameSecond(eventEnd, periodEnd)) {
        return true;
    }
    return false;
}

const MINUTES_IN_HOUR$1 = 60;
const HOURS_IN_DAY = 24;
/**
 * @param {?} __0
 * @return {?}
 */
function getDayView({ events = [], viewDate, hourSegments, dayStart, dayEnd, eventWidth, segmentHeight }) {
    if (!events) {
        events = [];
    }
    const /** @type {?} */ startOfView = setMinutes(setHours(startOfDay(viewDate), dayStart.hour), dayStart.minute);
    const /** @type {?} */ endOfView = setMinutes(setHours(startOfMinute(endOfDay(viewDate)), dayEnd.hour), dayEnd.minute);
    const /** @type {?} */ previousDayEvents = [];
    const /** @type {?} */ dayViewEvents = getEventsInPeriod({
        events: events.filter((event) => !event.allDay),
        periodStart: startOfView,
        periodEnd: endOfView
    })
        .sort((eventA, eventB) => {
        return eventA.start.valueOf() - eventB.start.valueOf();
    })
        .map((event) => {
        const /** @type {?} */ eventStart = event.start;
        const /** @type {?} */ eventEnd = event.end || eventStart;
        const /** @type {?} */ startsBeforeDay = eventStart < startOfView;
        const /** @type {?} */ endsAfterDay = eventEnd > endOfView;
        const /** @type {?} */ hourHeightModifier = hourSegments * segmentHeight / MINUTES_IN_HOUR$1;
        let /** @type {?} */ top = 0;
        if (eventStart > startOfView) {
            top += differenceInMinutes(eventStart, startOfView);
        }
        top *= hourHeightModifier;
        const /** @type {?} */ startDate = startsBeforeDay ? startOfView : eventStart;
        const /** @type {?} */ endDate = endsAfterDay ? endOfView : eventEnd;
        let /** @type {?} */ height = differenceInMinutes(endDate, startDate);
        if (!event.end) {
            height = segmentHeight;
        }
        else {
            height *= hourHeightModifier;
        }
        const /** @type {?} */ bottom = top + height;
        const /** @type {?} */ overlappingPreviousEvents = previousDayEvents.filter((previousEvent) => {
            const /** @type {?} */ previousEventTop = previousEvent.top;
            const /** @type {?} */ previousEventBottom = previousEvent.top + previousEvent.height;
            if (top < previousEventBottom && previousEventBottom < bottom) {
                return true;
            }
            else if (previousEventTop <= top && bottom <= previousEventBottom) {
                return true;
            }
            return false;
        });
        let /** @type {?} */ left = 0;
        while (overlappingPreviousEvents.some(previousEvent => previousEvent.left === left)) {
            left += eventWidth;
        }
        const /** @type {?} */ dayEvent = {
            event,
            height,
            width: eventWidth,
            top,
            left,
            startsBeforeDay,
            endsAfterDay
        };
        if (height > 0) {
            previousDayEvents.push(dayEvent);
        }
        return dayEvent;
    })
        .filter((dayEvent) => dayEvent.height > 0);
    const /** @type {?} */ width = Math.max(...dayViewEvents.map((event) => event.left + event.width));
    const /** @type {?} */ allDayEvents = getEventsInPeriod({
        events: events.filter((event) => event.allDay),
        periodStart: startOfDay(startOfView),
        periodEnd: endOfDay(endOfView)
    });
    return {
        events: dayViewEvents,
        width,
        allDayEvents
    };
}
/**
 * @param {?} __0
 * @return {?}
 */
function getDayViewHourGrid({ viewDate, hourSegments, dayStart, dayEnd }) {
    const /** @type {?} */ hours = [];
    const /** @type {?} */ startOfView = setMinutes(setHours(startOfDay(viewDate), dayStart.hour), dayStart.minute);
    const /** @type {?} */ endOfView = setMinutes(setHours(startOfMinute(endOfDay(viewDate)), dayEnd.hour), dayEnd.minute);
    const /** @type {?} */ segmentDuration = MINUTES_IN_HOUR$1 / hourSegments;
    const /** @type {?} */ startOfViewDay = startOfDay(viewDate);
    for (let /** @type {?} */ i = 0; i < HOURS_IN_DAY; i++) {
        const /** @type {?} */ segments = [];
        for (let /** @type {?} */ j = 0; j < hourSegments; j++) {
            const /** @type {?} */ date = addMinutes(addHours(startOfViewDay, i), j * segmentDuration);
            if (date >= startOfView && date < endOfView) {
                segments.push({
                    date,
                    isStart: j === 0
                });
            }
        }
        if (segments.length > 0) {
            hours.push({ segments });
        }
    }
    return hours;
}

const DAYS_IN_WEEK = 7;
const HOURS_IN_DAY$1 = 24;
let DAYS_OF_WEEK = {};
DAYS_OF_WEEK.SUNDAY = 0;
DAYS_OF_WEEK.MONDAY = 1;
DAYS_OF_WEEK.TUESDAY = 2;
DAYS_OF_WEEK.WEDNESDAY = 3;
DAYS_OF_WEEK.THURSDAY = 4;
DAYS_OF_WEEK.FRIDAY = 5;
DAYS_OF_WEEK.SATURDAY = 6;
DAYS_OF_WEEK[DAYS_OF_WEEK.SUNDAY] = "SUNDAY";
DAYS_OF_WEEK[DAYS_OF_WEEK.MONDAY] = "MONDAY";
DAYS_OF_WEEK[DAYS_OF_WEEK.TUESDAY] = "TUESDAY";
DAYS_OF_WEEK[DAYS_OF_WEEK.WEDNESDAY] = "WEDNESDAY";
DAYS_OF_WEEK[DAYS_OF_WEEK.THURSDAY] = "THURSDAY";
DAYS_OF_WEEK[DAYS_OF_WEEK.FRIDAY] = "FRIDAY";
DAYS_OF_WEEK[DAYS_OF_WEEK.SATURDAY] = "SATURDAY";
const DEFAULT_WEEKEND_DAYS = [
    DAYS_OF_WEEK.SUNDAY,
    DAYS_OF_WEEK.SATURDAY
];
/**
 * @param {?} __0
 * @return {?}
 */
function getWeekViewHeader({ viewDate, weekStartsOn, excluded = [], weekendDays = [] }) {
    const /** @type {?} */ start = startOfWeek(viewDate, { weekStartsOn });
    const /** @type {?} */ days = [];
    for (let /** @type {?} */ i = 0; i < DAYS_IN_WEEK; i++) {
        const /** @type {?} */ date = addDays(start, i);
        if (!excluded.some(e => date.getDay() === e)) {
            days.push(getWeekDay({ date, weekendDays }));
        }
    }
    return days;
}
/**
 * @param {?} __0
 * @return {?}
 */
function getWeekDay({ date, weekendDays = DEFAULT_WEEKEND_DAYS }) {
    const /** @type {?} */ today = startOfDay(new Date());
    return {
        date,
        isPast: date < today,
        isToday: isSameDay(date, today),
        isFuture: date > today,
        isWeekend: weekendDays.indexOf(getDay(date)) > -1
    };
}
/**
 * @param {?} __0
 * @return {?}
 */
function getMonthViewGrid({ events, viewDate, weekStartsOn, excluded = [], viewStart = startOfMonth(viewDate), viewEnd = endOfMonth(viewDate), weekendDays }) {
    let /** @type {?} */ start = startOfWeek(viewStart, { weekStartsOn });
    const /** @type {?} */ end = endOfWeek(viewEnd, { weekStartsOn });
    const /** @type {?} */ monthRows = (differenceInDays(end, start) + 1) / 7;
    let /** @type {?} */ width;
    let /** @type {?} */ left;
    const /** @type {?} */ weeks = [];
    for (let /** @type {?} */ week = 0; week < monthRows; week++) {
        const /** @type {?} */ endWeek = addDays(start, DAYS_IN_WEEK);
        const /** @type {?} */ eventsInWeek = [];
        const /** @type {?} */ eventsInPeriod = getEventsInPeriod({
            events,
            periodStart: start,
            periodEnd: endWeek
        }).map(event => {
            let /** @type {?} */ top = 28;
            let /** @type {?} */ hidden = false;
            if (differenceInHours(event.end, event.start) > differenceInHours(endWeek, event.start)) {
                width = (differenceInHours(endWeek, event.start) / 168) * 100;
            }
            else {
                width = (differenceInHours(event.end, event.start) / 168) * 100;
            }
            if (isBefore(event.start, start)) {
                left = 0;
                width = (differenceInHours(event.end, start) / 168) * 100;
            }
            else {
                left = (differenceInHours(event.start, start) / 168) * 100;
            }
            if (!event.allDay) {
                left = (differenceInHours(startOfDay(event.start), start) / 168) * 100;
            }
            if (width > 100) {
                width = 100;
            }
            top = getTop(event, eventsInWeek, top);
            if (top > 60) {
                hidden = true;
            }
            eventsInWeek.push({
                event: event,
                left: left,
                width: width,
                top: top,
                hidden: hidden
            });
            return {
                event: event,
                left: left,
                width: width,
                top: top,
                hidden: hidden
            };
        });
        let /** @type {?} */ previousDate = null;
        const /** @type {?} */ weekDays = [];
        for (let /** @type {?} */ i = 0; i < differenceInDays(endWeek, start); i++) {
            let /** @type {?} */ date;
            if (previousDate) {
                date = startOfDay(addHours(previousDate, HOURS_IN_DAY$1));
                if (previousDate.getTime() === date.getTime()) {
                    date = startOfDay(addHours(previousDate, HOURS_IN_DAY$1 + 1));
                }
                previousDate = date;
            }
            else {
                date = previousDate = start;
            }
            if (!excluded.some(e => date.getDay() === e)) {
                const /** @type {?} */ day = (getWeekDay({
                    date,
                    weekendDays: []
                }));
                const /** @type {?} */ dayEvents = getEventsInPeriod({
                    events,
                    periodStart: startOfDay(date),
                    periodEnd: endOfDay(date),
                });
                day.inMonth = isSameMonth(date, viewDate);
                day.badgeTotal = dayEvents.length;
                day.events = dayEvents;
                weekDays.push(day);
            }
        }
        weeks.push({
            events: eventsInPeriod,
            days: weekDays
        });
        start = addHours(start, (DAYS_IN_WEEK * HOURS_IN_DAY$1));
    }
    return {
        rowOffsets: weeks
    };
}
/**
 * @param {?} event
 * @param {?} events
 * @param {?} top
 * @return {?}
 */
function getTop(event, events, top) {
    const /** @type {?} */ over = [];
    const /** @type {?} */ tops = [28, 52, 76];
    for (const /** @type {?} */ e of events) {
        if ((isSameDay(event.start, e.event.start) || isSameDay(event.end, e.event.end)) ||
            isWithinRange(event.start, e.event.start, e.event.end) ||
            isWithinRange(event.end, e.event.start, e.event.end) ||
            isWithinRange(e.event.start, event.start, event.end) ||
            isWithinRange(e.event.end, event.start, event.end)) {
            top = e.top + 24;
            over.push(e);
        }
    }
    for (const /** @type {?} */ o of over) {
        for (const /** @type {?} */ t of tops) {
            if (o.top === t) {
                const /** @type {?} */ index = tops.indexOf(t);
                tops.splice(index, 1);
            }
        }
    }
    if (tops.length > 0) {
        top = tops[0];
    }
    return top;
}

const DAYS_IN_WEEK$1 = 7;
const HOURS_IN_DAY$2 = 24;
const MINUTES_IN_HOUR$2 = 60;
const SECONDS_IN_DAY = 60 * 60 * 24;
const SECONDS_IN_WEEK = SECONDS_IN_DAY * DAYS_IN_WEEK$1;

/**
 * @param {?} __0
 * @return {?}
 */
function getWeekView({ events = [], viewDate, hourSegments, dayStart, dayEnd, segmentHeight, weekStartsOn, }) {
    const /** @type {?} */ startOfViewWeek = startOfWeek(viewDate, { weekStartsOn });
    const /** @type {?} */ endOfViewWeek = endOfWeek(viewDate, { weekStartsOn });
    const /** @type {?} */ weekCols = differenceInDays(endOfViewWeek, startOfViewWeek) + 1;
    let /** @type {?} */ previousDate = startOfViewWeek;
    const /** @type {?} */ days = [];
    const /** @type {?} */ dayEvents = [];
    for (let /** @type {?} */ day = 0; day < weekCols; day++) {
        const /** @type {?} */ startDay = startOfDay(previousDate);
        const /** @type {?} */ endDay = endOfDay(previousDate);
        const /** @type {?} */ eventsInDay = getEventsInPeriod({
            events: events.filter((event) => !event.allDay),
            periodStart: startDay,
            periodEnd: endDay
        }).map(event => {
            let /** @type {?} */ top;
            let /** @type {?} */ height;
            if (isBefore(event.start, startDay)) {
                top = 0;
                height = differenceInMinutes(event.end, startDay);
            }
            else {
                top = differenceInMinutes(event.start, startDay);
                height = differenceInMinutes(event.end, event.start);
            }
            if (isAfter(event.end, endDay)) {
                height = differenceInMinutes(endDay, event.start);
            }
            dayEvents.push({
                event,
                top,
                height
            });
            return {
                event,
                top,
                height
            };
        });
        const /** @type {?} */ dayCols = differenceInHours(endDay, startDay) + 1;
        let /** @type {?} */ previousHour = startDay;
        const /** @type {?} */ hours = [];
        for (let /** @type {?} */ hour = 0; hour < dayCols; hour++) {
            const /** @type {?} */ eventsInHour = getEventsInPeriod({
                events,
                periodStart: startOfHour(previousHour),
                periodEnd: endOfHour(previousHour)
            });
            hours.push({
                hour: previousHour,
                events: eventsInHour
            });
            previousHour = startOfHour(addMinutes(previousHour, MINUTES_IN_HOUR$2 + 1));
        }
        days.push({
            date: previousDate,
            isToday: isSameDay(previousDate, new Date()),
            hours: hours,
            events: eventsInDay
        });
        previousDate = startOfDay(addHours(previousDate, HOURS_IN_DAY$2 + 1));
    }
    const /** @type {?} */ allDayWeekEvents = getEventsInPeriod({
        events: events.filter((event) => event.allDay),
        periodStart: startOfViewWeek,
        periodEnd: endOfViewWeek
    });
    const /** @type {?} */ allDayEventsMapped = allDayWeekEvents.map(event => {
        const /** @type {?} */ eventStart = event.start;
        const /** @type {?} */ eventEnd = event.end || eventStart;
        const /** @type {?} */ startsBeforeWeek = eventStart < startOfViewWeek;
        const /** @type {?} */ endsAfterWeek = eventEnd > endOfViewWeek;
        const /** @type {?} */ offset = getWeekViewEventOffset({
            event,
            startOfWeek: startOfViewWeek,
            excluded: [],
            precision: 'days'
        });
        const /** @type {?} */ span = getWeekViewEventSpan({
            event,
            offset,
            startOfWeekDate: startOfViewWeek,
            excluded: [],
            precision: 'days'
        });
        return {
            event: event,
            span: span,
            offset: offset,
            startsBeforeWeek: startsBeforeWeek,
            endsAfterWeek: endsAfterWeek
        };
    });
    return {
        days: days,
        allDays: allDayEventsMapped
    };
}
/**
 * @param {?} __0
 * @return {?}
 */
function getWeekViewEventSpan({ event, offset, startOfWeekDate, excluded, precision = 'days' }) {
    let /** @type {?} */ span = SECONDS_IN_DAY;
    const /** @type {?} */ begin = max(event.start, startOfWeekDate);
    if (event.end) {
        switch (precision) {
            case 'minutes':
                span = differenceInSeconds(event.end, begin);
                break;
            default:
                span = differenceInDays(addDays(event.end, 1), begin) * SECONDS_IN_DAY;
                break;
        }
    }
    const /** @type {?} */ offsetSeconds = offset * SECONDS_IN_DAY;
    const /** @type {?} */ totalLength = offsetSeconds + span;
    // the best way to detect if an event is outside the week-view
    // is to check if the total span beginning (from startOfWeekDay or event start) exceeds 7days
    if (totalLength > SECONDS_IN_WEEK) {
        span = SECONDS_IN_WEEK - offsetSeconds;
    }
    span -= getExcludedSeconds({
        startDate: begin,
        seconds: span,
        excluded,
        precision
    });
    return span / SECONDS_IN_DAY;
}
/**
 * @param {?} __0
 * @return {?}
 */
function getWeekViewEventOffset({ event, startOfWeek: startOfWeekDate, excluded = [], precision = 'days' }) {
    if (event.start < startOfWeekDate) {
        return 0;
    }
    let /** @type {?} */ offset = 0;
    switch (precision) {
        case 'days':
            offset =
                differenceInDays(startOfDay(event.start), startOfWeekDate) *
                    SECONDS_IN_DAY;
            break;
        case 'minutes':
            offset = differenceInSeconds(event.start, startOfWeekDate);
            break;
    }
    offset -= getExcludedSeconds({
        startDate: startOfWeekDate,
        seconds: offset,
        excluded,
        precision
    });
    return offset / SECONDS_IN_DAY;
}
/**
 * @param {?} __0
 * @return {?}
 */
function getExcludedSeconds({ startDate, seconds, excluded, precision = 'days' }) {
    if (excluded.length < 1) {
        return 0;
    }
    const /** @type {?} */ endDate = addSeconds(startDate, seconds - 1);
    const /** @type {?} */ dayStart = getDay(startDate);
    const /** @type {?} */ dayEnd = getDay(addSeconds(endDate, 0));
    let /** @type {?} */ result = 0; // Calculated in seconds
    let /** @type {?} */ current = startDate;
    while (current < endDate) {
        const /** @type {?} */ day = getDay(current);
        if (excluded.some(excludedDay => excludedDay === day)) {
            result += calculateExcludedSeconds({
                dayStart,
                dayEnd,
                day,
                precision,
                startDate,
                endDate
            });
        }
        current = addDays(current, 1);
    }
    return result;
}
/**
 * @param {?} __0
 * @return {?}
 */
function calculateExcludedSeconds({ precision, day, dayStart, dayEnd, startDate, endDate }) {
    if (precision === 'minutes') {
        if (day === dayStart) {
            return differenceInSeconds(endOfDay(startDate), startDate) + 1;
        }
        else if (day === dayEnd) {
            return differenceInSeconds(endDate, startOfDay(endDate)) + 1;
        }
    }
    return SECONDS_IN_DAY;
}

const HOURS_IN_DAY$3 = 24;
/**
 * @param {?} __0
 * @return {?}
 */
function getScheduleView({ events = [], viewDate }) {
    let /** @type {?} */ previousDate = viewDate;
    const /** @type {?} */ days = [];
    for (let /** @type {?} */ day = 0; day < 7; day++) {
        const /** @type {?} */ startDay = startOfDay(previousDate);
        const /** @type {?} */ endDay = endOfDay(previousDate);
        const /** @type {?} */ eventsInDay = getEventsInPeriod({
            events: events,
            periodStart: startDay,
            periodEnd: endDay
        });
        if (eventsInDay.length > 0) {
            days.push({
                date: previousDate,
                events: eventsInDay
            });
        }
        previousDate = startOfDay(addHours(previousDate, HOURS_IN_DAY$3 + 1));
    }
    return days;
}

class CalendarUtilsService {
    constructor() {
    }
    /**
     * @param {?} args
     * @return {?}
     */
    getScheduleView(args) {
        return getScheduleView(args);
    }
    /**
     * @param {?} args
     * @return {?}
     */
    getWeekView(args) {
        return getWeekView(args);
    }
    /**
     * @param {?} args
     * @return {?}
     */
    getMonthViewGrid(args) {
        return getMonthViewGrid(args);
    }
    /**
     * @param {?} args
     * @return {?}
     */
    getWeekViewHeader(args) {
        return getWeekViewHeader(args);
    }
    /**
     * @param {?} args
     * @return {?}
     */
    getDayView(args) {
        return getDayView(args);
    }
    /**
     * @param {?} args
     * @return {?}
     */
    getDayViewHourGrid(args) {
        return getDayViewHourGrid(args);
    }
}
CalendarUtilsService.decorators = [
    { type: Injectable },
];
/**
 * @nocollapse
 */
CalendarUtilsService.ctorParameters = () => [];

const validateEvents = (events) => {
    const /** @type {?} */ warn = (...args) => console.warn('angular-calendar', ...args);
    return validateEventsWithoutLog(events, warn);
};
/**
 * @param {?} outer
 * @param {?} inner
 * @return {?}
 */
function isInside(outer, inner) {
    return (outer.left <= inner.left &&
        inner.left <= outer.right &&
        outer.left <= inner.right &&
        inner.right <= outer.right &&
        outer.top <= inner.top &&
        inner.top <= outer.bottom &&
        outer.top <= inner.bottom &&
        inner.bottom <= outer.bottom);
}
/**
 * @param {?} events
 * @param {?} log
 * @return {?}
 */
function validateEventsWithoutLog(events, log) {
    let /** @type {?} */ isValid = true;
    /**
     * @param {?} msg
     * @param {?} event
     * @return {?}
     */
    function isError(msg, event) {
        log(msg, event);
        isValid = false;
    }
    if (!Array.isArray(events)) {
        log('Events must be an array', events);
        return false;
    }
    events.forEach(event => {
        if (!event.start) {
            isError('Event is missing the `start` property', event);
        }
        else if (!isDate(event.start)) {
            isError('Event `start` property should be a javascript date object. Do `new Date(event.start)` to fix it.', event);
        }
        if (event.end) {
            if (!isDate(event.end)) {
                isError('Event `end` property should be a javascript date object. Do `new Date(event.end)` to fix it.', event);
            }
            if (event.start > event.end) {
                isError('Event `start` property occurs after the `end`', event);
            }
        }
    });
    return isValid;
}

class CalendarDragHelper {
    /**
     * @param {?} dragContainerElement
     * @param {?} draggableElement
     */
    constructor(dragContainerElement, draggableElement) {
        this.dragContainerElement = dragContainerElement;
        this.startPosition = draggableElement.getBoundingClientRect();
    }
    /**
     * @param {?} __0
     * @return {?}
     */
    validateDrag({ x, y }) {
        const /** @type {?} */ newRect = Object.assign({}, this.startPosition, {
            left: this.startPosition.left + x,
            right: this.startPosition.right + x,
            top: this.startPosition.top + y,
            bottom: this.startPosition.bottom + y
        });
        return isInside(this.dragContainerElement.getBoundingClientRect(), newRect);
    }
}

class CalendarResizeHelper {
    /**
     * @param {?} resizeContainerElement
     * @param {?=} minWidth
     */
    constructor(resizeContainerElement, minWidth) {
        this.resizeContainerElement = resizeContainerElement;
        this.minWidth = minWidth;
    }
    /**
     * @param {?} __0
     * @return {?}
     */
    validateResize({ rectangle }) {
        if (this.minWidth && rectangle.width < this.minWidth) {
            return false;
        }
        return isInside(this.resizeContainerElement.getBoundingClientRect(), rectangle);
    }
}

/**
 * @hidden
 */
const MINUTES_IN_HOUR = 60;
class DayViewComponent {
    /**
     * @param {?} cdr
     * @param {?} utils
     * @param {?} locale
     */
    constructor(cdr, utils, locale) {
        this.cdr = cdr;
        this.utils = utils;
        /**
         * An array of events to display on view
         */
        this.events = [];
        /**
         * The number of segments in an hour. Must be <= 6
         */
        this.hourSegments = 2;
        /**
         * The height in pixels of each hour segment
         */
        this.hourSegmentHeight = 30;
        /**
         * The day start hours in 24 hour time. Must be 0-23
         */
        this.dayStartHour = 0;
        /**
         * The day start minutes. Must be 0-59
         */
        this.dayStartMinute = 0;
        /**
         * The day end hours in 24 hour time. Must be 0-23
         */
        this.dayEndHour = 23;
        /**
         * The day end minutes. Must be 0-59
         */
        this.dayEndMinute = 59;
        /**
         * The width in pixels of each event on the view
         */
        this.eventWidth = 150;
        /**
         * The grid size to snap resizing and dragging of events to
         */
        this.eventSnapSize = this.hourSegmentHeight;
        /**
         * Called when an event title is clicked
         */
        this.eventClicked = new EventEmitter();
        /**
         * Called when an hour segment is clicked
         */
        this.hourSegmentClicked = new EventEmitter();
        /**
         * Called when an event is resized or dragged and dropped
         */
        this.eventTimesChanged = new EventEmitter();
        /**
         * An output that will be called before the view is rendered for the current day.
         * If you add the `cssClass` property to a segment it will add that class to the hour segment in the template
         */
        this.beforeViewRender = new EventEmitter();
        /**
         * @hidden
         */
        this.hours = [];
        /**
         * @hidden
         */
        this.width = 0;
        /**
         * @hidden
         */
        this.currentResizes = new Map();
        this.locale = locale;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        if (this.refresh) {
            this.refreshSubscription = this.refresh.subscribe(() => {
                this.refreshAll();
                this.cdr.markForCheck();
            });
        }
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        if (changes.viewDate ||
            changes.dayStartHour ||
            changes.dayStartMinute ||
            changes.dayEndHour ||
            changes.dayEndMinute) {
            this.refreshHourGrid();
        }
        if (changes.events) {
            validateEvents(this.events);
        }
        if (changes.viewDate ||
            changes.events ||
            changes.dayStartHour ||
            changes.dayStartMinute ||
            changes.dayEndHour ||
            changes.dayEndMinute ||
            changes.eventWidth) {
            this.refreshView();
        }
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        if (this.refreshSubscription) {
            this.refreshSubscription.unsubscribe();
        }
    }
    /**
     * @param {?} dropEvent
     * @param {?} segment
     * @return {?}
     */
    eventDropped(dropEvent, segment) {
        if (dropEvent.dropData && dropEvent.dropData.event) {
            this.eventTimesChanged.emit({
                event: dropEvent.dropData.event,
                newStart: segment.date
            });
        }
    }
    /**
     * @param {?} event
     * @param {?} resizeEvent
     * @param {?} dayViewContainer
     * @return {?}
     */
    resizeStarted(event, resizeEvent, dayViewContainer) {
        this.currentResizes.set(event, {
            originalTop: event.top,
            originalHeight: event.height,
            edge: typeof resizeEvent.edges.top !== 'undefined' ? 'top' : 'bottom'
        });
        const /** @type {?} */ resizeHelper = new CalendarResizeHelper(dayViewContainer);
        this.validateResize = ({ rectangle }) => resizeHelper.validateResize({ rectangle });
        this.cdr.markForCheck();
    }
    /**
     * @param {?} event
     * @param {?} resizeEvent
     * @return {?}
     */
    resizing(event, resizeEvent) {
        const /** @type {?} */ currentResize = this.currentResizes.get(event);
        if (resizeEvent.edges.top) {
            event.top = currentResize.originalTop + +resizeEvent.edges.top;
            event.height = currentResize.originalHeight - +resizeEvent.edges.top;
        }
        else if (resizeEvent.edges.bottom) {
            event.height = currentResize.originalHeight + +resizeEvent.edges.bottom;
        }
    }
    /**
     * @param {?} dayEvent
     * @return {?}
     */
    resizeEnded(dayEvent) {
        const /** @type {?} */ currentResize = this.currentResizes.get(dayEvent);
        let /** @type {?} */ pixelsMoved;
        if (currentResize.edge === 'top') {
            pixelsMoved = dayEvent.top - currentResize.originalTop;
        }
        else {
            pixelsMoved = dayEvent.height - currentResize.originalHeight;
        }
        dayEvent.top = currentResize.originalTop;
        dayEvent.height = currentResize.originalHeight;
        const /** @type {?} */ pixelAmountInMinutes = MINUTES_IN_HOUR / (this.hourSegments * this.hourSegmentHeight);
        const /** @type {?} */ minutesMoved = pixelsMoved * pixelAmountInMinutes;
        let /** @type {?} */ newStart = dayEvent.event.start;
        let /** @type {?} */ newEnd = dayEvent.event.end;
        if (currentResize.edge === 'top') {
            newStart = addMinutes(newStart, minutesMoved);
        }
        else if (newEnd) {
            newEnd = addMinutes(newEnd, minutesMoved);
        }
        this.eventTimesChanged.emit({ newStart, newEnd, event: dayEvent.event });
        this.currentResizes.delete(dayEvent);
    }
    /**
     * @param {?} event
     * @param {?} dayViewContainer
     * @return {?}
     */
    dragStart(event, dayViewContainer) {
        const /** @type {?} */ dragHelper = new CalendarDragHelper(dayViewContainer, event);
        this.validateDrag = ({ x, y }) => this.currentResizes.size === 0 && dragHelper.validateDrag({ x, y });
        this.cdr.markForCheck();
    }
    /**
     * @param {?} dayEvent
     * @param {?} draggedInPixels
     * @return {?}
     */
    eventDragged(dayEvent, draggedInPixels) {
        const /** @type {?} */ pixelAmountInMinutes = MINUTES_IN_HOUR / (this.hourSegments * this.hourSegmentHeight);
        const /** @type {?} */ minutesMoved = draggedInPixels * pixelAmountInMinutes;
        const /** @type {?} */ newStart = addMinutes(dayEvent.event.start, minutesMoved);
        // possible issue
        let /** @type {?} */ newEnd = new Date();
        if (dayEvent.event.end) {
            newEnd = addMinutes(dayEvent.event.end, minutesMoved);
        }
        this.eventTimesChanged.emit({ newStart, newEnd, event: dayEvent.event });
    }
    /**
     * @return {?}
     */
    refreshHourGrid() {
        this.hours = this.utils.getDayViewHourGrid({
            viewDate: this.viewDate,
            hourSegments: this.hourSegments,
            dayStart: {
                hour: this.dayStartHour,
                minute: this.dayStartMinute
            },
            dayEnd: {
                hour: this.dayEndHour,
                minute: this.dayEndMinute
            }
        });
        this.beforeViewRender.emit({
            body: this.hours
        });
    }
    /**
     * @return {?}
     */
    refreshView() {
        this.view = this.utils.getDayView({
            events: this.events,
            viewDate: this.viewDate,
            hourSegments: this.hourSegments,
            dayStart: {
                hour: this.dayStartHour,
                minute: this.dayStartMinute
            },
            dayEnd: {
                hour: this.dayEndHour,
                minute: this.dayEndMinute
            },
            eventWidth: this.eventWidth,
            segmentHeight: this.hourSegmentHeight
        });
    }
    /**
     * @return {?}
     */
    refreshAll() {
        this.refreshHourGrid();
        this.refreshView();
    }
}
DayViewComponent.decorators = [
    { type: Component, args: [{
                selector: 'calendar-day-view',
                template: `
    <div class="cal-day-view" #dayViewContainer>
      <calendar-day-view-all-day-event
        *ngFor="let event of view.allDayEvents"
        [event]="event"
        (eventClicked)="eventClicked.emit({event: event})">
      </calendar-day-view-all-day-event>
      <div class="cal-hour-rows">
        <div class="cal-events">
          <div
            #event
            *ngFor="let dayEvent of view?.events"
            class="cal-event-container"
            [class.cal-draggable]="dayEvent.event.draggable"
            [class.cal-starts-within-day]="!dayEvent.startsBeforeDay"
            [class.cal-ends-within-day]="!dayEvent.endsAfterDay"
            [ngClass]="dayEvent.event.cssClass"
            mwlResizable
            [resizeEdges]="{top: dayEvent.event?.resizable?.beforeStart, bottom: dayEvent.event?.resizable?.afterEnd}"
            [resizeSnapGrid]="{top: eventSnapSize, bottom: eventSnapSize}"
            [validateResize]="validateResize"
            (resizeStart)="resizeStarted(dayEvent, $event, dayViewContainer)"
            (resizing)="resizing(dayEvent, $event)"
            (resizeEnd)="resizeEnded(dayEvent)"
            mwlDraggable
            [dragAxis]="{x: false, y: dayEvent.event.draggable && currentResizes.size === 0}"
            [dragSnapGrid]="{y: eventSnapSize}"
            [validateDrag]="validateDrag"
            (dragStart)="dragStart(event, dayViewContainer)"
            (dragEnd)="eventDragged(dayEvent, $event.y)"
            [style.marginTop.px]="dayEvent.top"
            [style.height.px]="dayEvent.height"
            [style.marginLeft.px]="dayEvent.left + 70"
            [style.width.px]="dayEvent.width - 1">
            <calendar-day-view-event
              [dayEvent]="dayEvent"
              (eventClicked)="eventClicked.emit({event: dayEvent.event})">
            </calendar-day-view-event>
          </div>
        </div>
        <div class="cal-hour" *ngFor="let hour of hours" [style.minWidth.px]="view?.width + 70">
          <calendar-day-view-hour-segment
            *ngFor="let segment of hour.segments"
            [style.height.px]="hourSegmentHeight"
            [segment]="segment"
            [segmentHeight]="hourSegmentHeight"
            [locale]="locale"
            (click)="hourSegmentClicked.emit({date: segment.date})">
          </calendar-day-view-hour-segment>
        </div>
      </div>
    </div>
  `,
                styles: [`
    .cal-day-view .cal-hour-rows {
      width: 100%;
      border: solid 1px #e1e1e1;
      overflow-x: scroll;
      position: relative;
    }
  
    .cal-day-view .cal-hour {
      background-color: white;
    }
  
    .cal-event-container {
      position: absolute;
    }
  
  
  `]
            },] },
];
/**
 * @nocollapse
 */
DayViewComponent.ctorParameters = () => [
    { type: ChangeDetectorRef, },
    { type: CalendarUtilsService, },
    { type: undefined, decorators: [{ type: Inject, args: [LOCALE_ID,] },] },
];
DayViewComponent.propDecorators = {
    'viewDate': [{ type: Input },],
    'events': [{ type: Input },],
    'hourSegments': [{ type: Input },],
    'hourSegmentHeight': [{ type: Input },],
    'dayStartHour': [{ type: Input },],
    'dayStartMinute': [{ type: Input },],
    'dayEndHour': [{ type: Input },],
    'dayEndMinute': [{ type: Input },],
    'eventWidth': [{ type: Input },],
    'refresh': [{ type: Input },],
    'locale': [{ type: Input },],
    'eventSnapSize': [{ type: Input },],
    'eventClicked': [{ type: Output },],
    'hourSegmentClicked': [{ type: Output },],
    'eventTimesChanged': [{ type: Output },],
    'beforeViewRender': [{ type: Output },],
};

class DayViewAllDayEventComponent {
    constructor() {
        this.eventClicked = new EventEmitter();
    }
}
DayViewAllDayEventComponent.decorators = [
    { type: Component, args: [{
                selector: 'calendar-day-view-all-day-event',
                template: `
    <ng-template
    #defaultTemplate
    let-event="event"
    let-eventClicked="eventClicked">
    <div
      class="cal-all-day-event"
      [style.backgroundColor]="event.color ? event.color.primary : event.calendar.color">
      <calendar-event-action [event]="event"></calendar-event-action>
      <calendar-event-title
        [event]="event"
        view="day"
        (mwlClick)="eventClicked.emit()">
      </calendar-event-title>
    </div>
    </ng-template>
    <ng-template
    [ngTemplateOutlet]="defaultTemplate"
    [ngTemplateOutletContext]="{
      event: event,
      eventClicked: eventClicked
    }">
    </ng-template>
  `,
                styles: [`
    .cal-all-day-event{
        margin-bottom: 2px; 
        font-size: 13px;
        padding: 5px;
    }
  `]
            },] },
];
/**
 * @nocollapse
 */
DayViewAllDayEventComponent.ctorParameters = () => [];
DayViewAllDayEventComponent.propDecorators = {
    'event': [{ type: Input },],
    'customTemplate': [{ type: Input },],
    'eventTitleTemplate': [{ type: Input },],
    'eventClicked': [{ type: Output },],
};

class DayViewEventComponent {
    constructor() {
        this.eventClicked = new EventEmitter();
    }
}
DayViewEventComponent.decorators = [
    { type: Component, args: [{
                selector: 'calendar-day-view-event',
                template: `
    <ng-template
    #defaultTemplate
    let-dayEvent="dayEvent"
    let-eventClicked="eventClicked">
    <div
      class="cal-event"
      [style.backgroundColor]="dayEvent.event.color ? dayEvent.event.color.primary : dayEvent.event.calendar.color">
      <calendar-event-action [event]="dayEvent.event"></calendar-event-action>
      <calendar-event-title
        [event]="dayEvent.event"
        view="day"
        (click)="eventClicked.emit()">
      </calendar-event-title>
    </div>
    </ng-template>
    <ng-template
    [ngTemplateOutlet]="defaultTemplate"
    [ngTemplateOutletContext]="{
      dayEvent: dayEvent,
      eventClicked: eventClicked
    }">
    </ng-template>
  `,
                styles: [`
    .cal-event{
        height: 100%;
        font-size: 12px;
        padding-left: 5px;
        color: white !important;
        font-size: 12px;
    }
    .cal-event a {
        color: white !important;
        font-size: 12px; 
    }
  `]
            },] },
];
/**
 * @nocollapse
 */
DayViewEventComponent.ctorParameters = () => [];
DayViewEventComponent.propDecorators = {
    'dayEvent': [{ type: Input },],
    'tooltipPlacement': [{ type: Input },],
    'tooltipAppendToBody': [{ type: Input },],
    'customTemplate': [{ type: Input },],
    'eventTitleTemplate': [{ type: Input },],
    'tooltipTemplate': [{ type: Input },],
    'eventClicked': [{ type: Output },],
};

class DayViewHourSegmentComponent {
}
DayViewHourSegmentComponent.decorators = [
    { type: Component, args: [{
                selector: 'calendar-day-view-hour-segment',
                template: `
    <ng-template
    #defaultTemplate
    let-segment="segment"
    let-locale="locale">
    <div
      class="cal-hour-segment"
      [style.height.px]="segmentHeight"
      [class.cal-hour-start]="segment.isStart"
      [class.cal-after-hour-start]="!segment.isStart"
      [ngClass]="segment.cssClass">
      <div class="cal-time">
        {{ segment.date | date: 'h:mm a' }}
      </div>
    </div>
    </ng-template>
    <ng-template
    [ngTemplateOutlet]="defaultTemplate"
    [ngTemplateOutletContext]="{
      segment: segment,
      locale: locale
    }">
    </ng-template>
  `,
                styles: [`
    .cal-hour-segment{
        display: -webkit-box;
        display: -ms-flexbox;
        display: flex;
        border-bottom: 1px solid #e1e1e1;
        border-right: 1px solid #e1e1e1;
        border-left: 1px solid #e1e1e1;
    }
    .cal-hour-segment.cal-after-hour-start .cal-time {
        color: transparent;
        border-bottom: 1px solid #e1e1e1;

    }
    .cal-hour-segment .cal-time {    
        height: 30px;
        font-size: 12px;
        padding-left: 15px;
        border-right: 1px solid #e1e1e1;
    }


    .cal-hour-segment .cal-hour-segment-header:not(:last-child) {
        border-right: 1px solid #e1e1e1;
      }
    .cal-hour-segment .cal-time {
        position: absolute;
        width: 70px;
        left: 0;
    }
    .cal-hour-segment .cal-hour-segment-header {
        -webkit-box-flex: 1;
            -ms-flex: 1;
                flex: 1;
    } 
    .cal-hour-segment:hover {
        background-color: transparent;
    }

    .cal-hour-segment.cal-hour-start {
        border-bottom: none;
    }
  `]
            },] },
];
/**
 * @nocollapse
 */
DayViewHourSegmentComponent.ctorParameters = () => [];
DayViewHourSegmentComponent.propDecorators = {
    'segment': [{ type: Input },],
    'segmentHeight': [{ type: Input },],
    'locale': [{ type: Input },],
    'customTemplate': [{ type: Input },],
};

class EventActionComponent {
    constructor() { }
    /**
     * @return {?}
     */
    ngOnInit() {
    }
}
EventActionComponent.decorators = [
    { type: Component, args: [{
                selector: 'calendar-event-action',
                template: `
    <span *ngIf="event.actions" class="cal-event-actions">
      <a
        class="cal-event-action"
        href="javascript:;"
        *ngFor="let action of event.actions"
        (mwlClick)="action.onClick({event: event})"
        [ngClass]="action.cssClass"
        [innerHtml]="action.label">
      </a>
    </span>
  `,
                styles: [`

  `]
            },] },
];
/**
 * @nocollapse
 */
EventActionComponent.ctorParameters = () => [];
EventActionComponent.propDecorators = {
    'event': [{ type: Input },],
};

class EventTitleComponent {
    constructor() { }
    /**
     * @return {?}
     */
    ngOnInit() {
    }
}
EventTitleComponent.decorators = [
    { type: Component, args: [{
                selector: 'calendar-event-title',
                template: `
    <ng-template
    #defaultTemplate
    let-event="event"
    let-view="view">
    <a
      class="cal-event-title"
      href="javascript:;"
      [innerHTML]="event.title">
    </a>
    </ng-template>
    <ng-template
    [ngTemplateOutlet]="defaultTemplate"
    [ngTemplateOutletContext]="{
      event: event,
      view: view
    }">
    </ng-template>
  `,
                styles: [`
    .cal-event-title{
        font-size: 12px;
        color: white;
    }
  `]
            },] },
];
/**
 * @nocollapse
 */
EventTitleComponent.ctorParameters = () => [];
EventTitleComponent.propDecorators = {
    'event': [{ type: Input },],
    'customTemplate': [{ type: Input },],
    'view': [{ type: Input },],
};

class DayModule {
}
DayModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                    ResizableModule,
                    DragAndDropModule
                ],
                declarations: [
                    DayViewComponent,
                    DayViewAllDayEventComponent,
                    DayViewEventComponent,
                    DayViewHourSegmentComponent,
                    EventActionComponent,
                    EventTitleComponent
                ],
                exports: [
                    DayViewComponent,
                ]
            },] },
];
/**
 * @nocollapse
 */
DayModule.ctorParameters = () => [];

class MonthViewComponent {
    /**
     * @hidden
     * @param {?} cdr
     * @param {?} utils
     * @param {?} locale
     */
    constructor(cdr, utils, locale) {
        this.cdr = cdr;
        this.utils = utils;
        /**
         * An array of events to display on view.
         */
        this.events = [];
        /**
         * An array of day indexes (0 = sunday, 1 = monday etc) that will be hidden on the view
         */
        this.excludeDays = [];
        /**
         * Whether the events list for the day of the `viewDate` option is visible or not
         */
        this.activeDayIsOpen = false;
        /**
         * An output that will be called before the view is rendered for the current month.
         * If you add the `cssClass` property to a day in the body it will add that class to the cell element in the template
         */
        this.beforeViewRender = new EventEmitter();
        /**
         * Called when the day cell is clicked
         */
        this.dayClicked = new EventEmitter();
        /**
         * Called when the event title is clicked
         */
        this.eventClicked = new EventEmitter();
        /**
         * Called when an event is dragged and dropped
         */
        this.eventTimesChanged = new EventEmitter();
        this.locale = locale;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        if (this.refresh) {
            this.refreshSubscription = this.refresh.subscribe(() => {
                this.refreshAll();
                this.cdr.markForCheck();
            });
        }
    }
    /**
     * @hidden
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        if (changes.viewDate || changes.excludeDays || changes.weekendDays) {
            this.refreshHeader();
        }
        if (changes.events) {
            validateEvents(this.events);
        }
        if (changes.viewDate ||
            changes.events ||
            changes.excludeDays ||
            changes.weekendDays) {
            this.refreshBody();
        }
        if (changes.activeDayIsOpen ||
            changes.viewDate ||
            changes.events ||
            changes.excludeDays) {
            this.checkActiveDayIsOpen();
        }
    }
    /**
     * @hidden
     * @return {?}
     */
    ngOnDestroy() {
        if (this.refreshSubscription) {
            this.refreshSubscription.unsubscribe();
        }
    }
    /**
     * @return {?}
     */
    refreshHeader() {
        this.columnHeaders = this.utils.getWeekViewHeader({
            viewDate: this.viewDate,
            weekStartsOn: this.weekStartsOn,
            excluded: this.excludeDays,
            weekendDays: this.weekendDays
        });
        this.emitBeforeViewRender();
    }
    /**
     * @return {?}
     */
    refreshBody() {
        this.view = this.utils.getMonthViewGrid({
            events: this.events,
            viewDate: this.viewDate,
            weekStartsOn: this.weekStartsOn,
            excluded: this.excludeDays,
            weekendDays: this.weekendDays
        });
        this.emitBeforeViewRender();
    }
    /**
     * @return {?}
     */
    emitBeforeViewRender() {
        if (this.columnHeaders && this.view) {
            this.beforeViewRender.emit({
                header: this.columnHeaders,
                body: this.view.days
            });
        }
    }
    /**
     * @return {?}
     */
    checkActiveDayIsOpen() {
        if (this.activeDayIsOpen === true) {
            this.openDay = this.view.days.find((day) => isSameDay(day.date, this.viewDate));
            const /** @type {?} */ index = this.view.days.indexOf(this.openDay);
            this.openRowIndex =
                Math.floor(index / this.view.totalDaysVisibleInWeek) *
                    this.view.totalDaysVisibleInWeek;
        }
        else {
            this.openRowIndex = null;
            this.openDay = null;
        }
    }
    /**
     * @return {?}
     */
    refreshAll() {
        this.columnHeaders = null;
        this.view = null;
        this.refreshHeader();
        this.refreshBody();
        this.checkActiveDayIsOpen();
    }
}
MonthViewComponent.decorators = [
    { type: Component, args: [{
                selector: 'calendar-month-view',
                template: `
    <div class="cal-month-view">
        <calendar-month-view-header
          [days]="columnHeaders"
          [locale]="locale">
        </calendar-month-view-header>
        <div class="cal-days">
            <div class="cal-row" *ngFor="let rowIndex of view.rowOffsets; let index = index">
              <div *ngIf="rowIndex" class="cal-cell-row">
                  <calendar-month-cell
                  class="cal-day-cell"
                  *ngFor="let day of rowIndex.days"
                  [class.cal-drag-over]="day.dragOver"
                  [ngClass]="{'isToday': day.isToday }"
                  [day]="day"
                  [openDay]="openDay"
                  [locale]="locale"
                  (click)="dayClicked.emit({day:day})"
                  mwlDroppable
                  (dragEnter)="day.dragOver = true"
                  (dragLeave)="day.dragOver = false">
                </calendar-month-cell>
                <div *ngFor="let event of rowIndex.events">
                    <div
                    *ngIf="!event.hidden && event.event.allDay"
                    class="month-event"
                    [style.height.px]="20"
                    [style.top.px]="event.top"
                    [style.backgroundColor]="event.event.color ? event.event.color.primary : event.event.calendar.color"
                    [style.left]="event.left + '%'"
                    [style.width]="event.width + '%'"
                    (click)="eventClicked.emit({event:event.event})"
                    >
                    <div class="month-event-title">
                        {{ event.event.start | date : 'hh:mm a'}} {{ event.event.title }}
                    </div>
                    </div>
                </div>
              </div>
            </div>
          </div>
    </div>
  
  `,
                styles: [`
    .cal-month-view {
      overflow: hidden;
      background-color: white; }

    .cal-days {
      border: 1px solid #E0E0E0;
      border-bottom: 0;
      position: relative; }

    .month-events-of-week {
      -webkit-box-flex: 1;
          -ms-flex: 1;
              flex: 1; }

    .cal-row {
      position: relative; }

    .month-event {
      position: absolute;
      color: white;
      padding-left: 5px;
      padding-right: 5px;
      font-size: 12px;
      display: -webkit-box;
      display: -ms-flexbox;
      display: flex;
      -webkit-box-align: center;
          -ms-flex-align: center;
              align-items: center;
      cursor: pointer;
      overflow: hidden; }

    .month-event-title {
      white-space: nowrap;
      text-overflow: ellipsis;
      -webkit-box-flex: 1;
          -ms-flex: 1;
              flex: 1;
      overflow: hidden; }

    .non-all-day-event {
      cursor: pointer;
      position: absolute;
      display: -webkit-box;
      display: -ms-flexbox;
      display: flex;
      font-size: 12px;
      -webkit-box-align: center;
          -ms-flex-align: center;
              align-items: center;
      width: 14.28571%;
      overflow: hidden; }

    .non-all-day-event-title {
      white-space: nowrap;
      text-overflow: ellipsis;
      -webkit-box-flex: 1;
          -ms-flex: 1;
              flex: 1;
      overflow: hidden; }

    .non-all-day-event-bg {
      height: 10px;
      width: 10px;
      border-radius: 50%;
      margin-left: 5px;
      margin-right: 5px; }

    .cal-cell-row:hover {
      background-color: #fafafa; }

    .cal-cell-row .cal-cell:hover,
    .cal-cell.cal-has-events.cal-open {
      background-color: #ededed; }

    .cal-cell-row {
      display: -webkit-box;
      display: -ms-flexbox;
      display: flex; }

    .cal-cell {
      float: left;
      -webkit-box-flex: 1;
          -ms-flex: 1;
              flex: 1;
      display: -webkit-box;
      display: -ms-flexbox;
      display: flex;
      -webkit-box-orient: vertical;
      -webkit-box-direction: normal;
          -ms-flex-direction: column;
              flex-direction: column;
      -webkit-box-align: stretch;
          -ms-flex-align: stretch;
              align-items: stretch; }

    .cal-day-cell {
      height: 100px;
      width: 14.28571%;
      overflow: hidden;
      position: relative; }

    .cal-day-cell:not(:last-child) {
      border-right: 1px solid #E0E0E0; }

    .cal-weekend:not(:first-child) {
      border-right: none !important; }

    .cal-days .cal-cell-row {
      border-bottom: 1px solid #E0E0E0; }

    .isToday {
      background-color: #fafafa; }
  `]
            },] },
];
/**
 * @nocollapse
 */
MonthViewComponent.ctorParameters = () => [
    { type: ChangeDetectorRef, },
    { type: CalendarUtilsService, },
    { type: undefined, decorators: [{ type: Inject, args: [LOCALE_ID,] },] },
];
MonthViewComponent.propDecorators = {
    'viewDate': [{ type: Input },],
    'events': [{ type: Input },],
    'excludeDays': [{ type: Input },],
    'activeDayIsOpen': [{ type: Input },],
    'refresh': [{ type: Input },],
    'locale': [{ type: Input },],
    'weekStartsOn': [{ type: Input },],
    'weekendDays': [{ type: Input },],
    'beforeViewRender': [{ type: Output },],
    'dayClicked': [{ type: Output },],
    'eventClicked': [{ type: Output },],
    'eventTimesChanged': [{ type: Output },],
};

class MonthCellComponent {
    constructor() {
        this.highlightDay = new EventEmitter();
        this.unhighlightDay = new EventEmitter();
        this.eventClicked = new EventEmitter();
    }
    /**
     * @hidden
     * @param {?} mouseEvent
     * @param {?} calendarEvent
     * @return {?}
     */
    onEventClick(mouseEvent, calendarEvent) {
        if (mouseEvent.stopPropagation) {
            mouseEvent.stopPropagation();
        }
        this.eventClicked.emit({ event: calendarEvent });
    }
    /**
     * @param {?} end
     * @param {?} start
     * @return {?}
     */
    isSameDay(end, start) {
        const /** @type {?} */ diff = isSameDay(end, start);
        const /** @type {?} */ viewEnd = endOfWeek(start);
        return diff;
    }
}
MonthCellComponent.decorators = [
    { type: Component, args: [{
                selector: 'calendar-month-cell',
                template: `
    <ng-template
    #defaultTemplate
    let-day="day"
    let-openDay="openDay"
    let-locale="locale">
    <div class="cal-cell">
      <div class="cal-cell-top">
          <span class="cal-day-number">{{ day.date | date: 'dd' }}</span>
      </div>
      <div class="cal-cell-day-events">
        <div *ngFor="let event of day.events">
          <div class="cal-cell-day-event"
          [style.height.px]="28"
          *ngIf="!event.allDay">
              <div
              class="cal-cell-day-event-bg"
              [style.backgroundColor]="event.color ? event.color.primary : event.calendar.color">
              </div>
              <div
              *ngIf="!isSameDay(day.date, event.start)"
              class="cal-cell-day-event-title">
                  {{ day.date | date : 'hh:mm a'}} {{ event.title }}                  
              </div>
              <div 
              *ngIf="isSameDay(day.date, event.start)"
                class="cal-cell-day-event-title">
                    {{ event.start | date : 'hh:mm a'}} {{ event.title }}    
              </div>
          </div>
        </div>
      </div>
      <span class="cal-day-badge" *ngIf="day.badgeTotal > 2">{{ day.badgeTotal - 2 }} more ...</span>
    </div>
    </ng-template>
    <ng-template
    [ngTemplateOutlet]="defaultTemplate"
    [ngTemplateOutletContext]="{
      day: day,
      openDay: openDay,
      locale: locale,
      highlightDay: highlightDay,
      unhighlightDay: unhighlightDay,
      eventClicked: eventClicked
    }">
    </ng-template>
  `,
                styles: [`
    .cal-cell {
        position: relative;
        height: 100px;
    }
    .cal-cell-top{
        z-index: 100;
        display: -webkit-box;
        display: -ms-flexbox;
        display: flex;
        -webkit-box-pack: end;
            -ms-flex-pack: end;
                justify-content: flex-end;
        height: 24px;
        -webkit-box-orient: horizontal;
        -webkit-box-direction: normal;
            -ms-flex-direction: row;
                flex-direction: row;
    }

    .cal-day-badge {
        background-color: #e9e9e9;
        padding: 3px 7px;
        font-size: 12px;
        line-height: 1 !important;
        color: rgb(121, 121, 121);
        text-align: center;
        white-space: nowrap;
        vertical-align: middle;
        position: absolute;
        bottom: 0;
        width: 100%;
    }

    .cal-day-number {
        -webkit-box-flex: 1;
            -ms-flex: 1;
                flex: 1;
        font-size: 0.8em;
        margin-left: 5px;
        margin-right: 15px;
        margin-bottom: 10px;
        color: #555555;
    }
    .cal-cell-day-event {
        display: -webkit-box;
        display: -ms-flexbox;
        display: flex;
        font-size: 12px;
        -webkit-box-align: center;
            -ms-flex-align: center;
                align-items: center;
        cursor: pointer;
        overflow: hidden;
    }
    .cal-cell-day-event-bg{
        height: 10px;
        width: 10px;
        border-radius: 50%;
        margin-left: 5px;
        margin-right: 5px;
    }

    .cal-cell-day-event-title{
        white-space: nowrap;
        text-overflow: ellipsis;
        -webkit-box-flex: 1;
            -ms-flex: 1;
                flex: 1;
        overflow: hidden;
    }
  `],
                // tslint:disable-next-line:use-host-property-decorator
                host: {
                    class: 'cal-cell cal-day-cell',
                    '[class.cal-past]': 'day.isPast',
                    '[class.cal-today]': 'day.isToday',
                    '[class.cal-future]': 'day.isFuture',
                    '[class.cal-weekend]': 'day.isWeekend',
                    '[class.cal-in-month]': 'day.inMonth',
                    '[class.cal-out-month]': '!day.inMonth',
                    '[class.cal-has-events]': 'day.events.length > 0',
                    '[class.cal-open]': 'day === openDay',
                    '[style.backgroundColor]': 'day.backgroundColor'
                }
            },] },
];
/**
 * @nocollapse
 */
MonthCellComponent.ctorParameters = () => [];
MonthCellComponent.propDecorators = {
    'day': [{ type: Input },],
    'openDay': [{ type: Input },],
    'locale': [{ type: Input },],
    'tooltipPlacement': [{ type: Input },],
    'tooltipAppendToBody': [{ type: Input },],
    'customTemplate': [{ type: Input },],
    'tooltipTemplate': [{ type: Input },],
    'highlightDay': [{ type: Output },],
    'unhighlightDay': [{ type: Output },],
    'eventClicked': [{ type: Output },],
};

class MonthViewHeaderComponent {
}
MonthViewHeaderComponent.decorators = [
    { type: Component, args: [{
                selector: 'calendar-month-view-header',
                template: `
    <ng-template
    #defaultTemplate
    let-days="days"
    let-locale="locale">
    <div class="cal-cell-row cal-header">
      <div
        class="cal-cell"
        *ngFor="let day of days"
        [class.cal-past]="day.isPast"
        [class.cal-today]="day.isToday"
        [class.cal-future]="day.isFuture"
        [class.cal-weekend]="day.isWeekend"
        [ngClass]="day.cssClass">
        {{ day.date | date: 'EE' }}
      </div>
    </div>
    </ng-template>
    <ng-template
    [ngTemplateOutlet]="customTemplate || defaultTemplate"
    [ngTemplateOutletContext]="{days: days, locale: locale}">
    </ng-template>
  `,
                styles: [`
    .cal-header {
        text-align: center;
        font-weight: bolder;
        padding-top: 5px;
        padding-bottom: 5px;
        border: 1px solid #e1e1e1;
      }

      .cal-cell-row:hover {
        background-color: #fafafa;
      }

      .cal-header .cal-cell {
        padding: 5px 0;
        overflow: hidden;
        text-overflow: ellipsis;
        display: block;
        white-space: nowrap;
        font-size: 12px;
      }

      .cal-cell-row {
        display: -webkit-box;
        display: -ms-flexbox;
        display: flex;
      }
    
      .cal-cell {
        float: left;
        -webkit-box-flex: 1;
            -ms-flex: 1;
                flex: 1;
        display: -webkit-box;
        display: -ms-flexbox;
        display: flex;
        -webkit-box-orient: vertical;
        -webkit-box-direction: normal;
            -ms-flex-direction: column;
                flex-direction: column;
        -webkit-box-align: stretch;
            -ms-flex-align: stretch;
                align-items: stretch;
      }

      .cal-cell-row {
        border-bottom: 1px solid #e1e1e1;
      }
  `]
            },] },
];
/**
 * @nocollapse
 */
MonthViewHeaderComponent.ctorParameters = () => [];
MonthViewHeaderComponent.propDecorators = {
    'days': [{ type: Input },],
    'locale': [{ type: Input },],
    'customTemplate': [{ type: Input },],
};

class OpenDayEventsComponent {
    constructor() { }
    /**
     * @return {?}
     */
    ngOnInit() {
    }
}
OpenDayEventsComponent.decorators = [
    { type: Component, args: [{
                selector: 'calendar-open-day-events',
                template: `
    <p>
      open-day-events works!
    </p>
  `,
                styles: [`

  `]
            },] },
];
/**
 * @nocollapse
 */
OpenDayEventsComponent.ctorParameters = () => [];

class MonthModule {
}
MonthModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                    ResizableModule,
                    DragAndDropModule
                ],
                declarations: [
                    MonthViewComponent,
                    MonthCellComponent,
                    MonthViewHeaderComponent,
                    OpenDayEventsComponent
                ],
                exports: [
                    MonthViewComponent
                ]
            },] },
];
/**
 * @nocollapse
 */
MonthModule.ctorParameters = () => [];

class WeekViewComponent {
    /**
     * @param {?} cdr
     * @param {?} utils
     */
    constructor(cdr, utils) {
        this.cdr = cdr;
        this.utils = utils;
        /**
         * An array of events to display on view
         */
        this.events = [];
        /**
         * An array of day indexes (0 = sunday, 1 = monday etc) that will be hidden on the view
         */
        this.excludeDays = [];
        /**
         * @hidden
         */
        this.hours = [];
        /**
         * The precision to display events.
         * `days` will round event start and end dates to the nearest day and `minutes` will not do this rounding
         */
        this.precision = 'days';
        /**
         * The number of segments in an hour. Must be <= 6
         */
        this.hourSegments = 2;
        /**
         * The height in pixels of each hour segment
         */
        this.hourSegmentHeight = 30;
        /**
         * The day start hours in 24 hour time. Must be 0-23
         */
        this.dayStartHour = 0;
        /**
         * The day start minutes. Must be 0-59
         */
        this.dayStartMinute = 0;
        /**
         * The day end hours in 24 hour time. Must be 0-23
         */
        this.dayEndHour = 23;
        /**
         * The day end minutes. Must be 0-59
         */
        this.dayEndMinute = 59;
        /**
         * The width in pixels of each event on the view
         */
        this.eventWidth = 150;
        /**
         * An output that will be called before the view is rendered for the current week.
         * If you add the `cssClass` property to a day in the header it will add that class to the cell element in the template
         */
        this.beforeViewRender = new EventEmitter();
        /**
         * Called when the event title is clicked
         */
        this.eventClicked = new EventEmitter();
        this.currentTime = this.getCurrentTime();
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        if (this.refresh) {
            this.refreshSubscription = this.refresh.subscribe(() => {
                this.refreshAll();
                this.cdr.markForCheck();
            });
        }
    }
    /**
     * @hidden
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        if (changes.viewDate || changes.excludeDays || changes.weekendDays) {
            this.refreshHeader();
        }
        if (changes.events || changes.viewDate || changes.excludeDays) {
            this.refreshBody();
        }
    }
    /**
     * @return {?}
     */
    getCurrentTime() {
        const /** @type {?} */ time = new Date();
        const /** @type {?} */ top = differenceInMinutes(time, startOfDay(time));
        return top;
    }
    /**
     * @return {?}
     */
    refreshBody() {
        this.view = this.utils.getWeekView({
            events: this.events,
            viewDate: this.viewDate,
            hourSegments: this.hourSegments,
            dayStart: {
                hour: this.dayStartHour,
                minute: this.dayStartMinute
            },
            dayEnd: {
                hour: this.dayEndHour,
                minute: this.dayEndMinute
            },
            segmentHeight: this.hourSegmentHeight,
            weekStartsOn: this.weekStartsOn,
        });
    }
    /**
     * @return {?}
     */
    refreshHeader() {
        this.days = this.utils.getWeekViewHeader({
            viewDate: this.viewDate,
            weekStartsOn: this.weekStartsOn,
            excluded: this.excludeDays,
            weekendDays: this.weekendDays
        });
        this.hours = this.utils.getDayViewHourGrid({
            viewDate: this.viewDate,
            hourSegments: this.hourSegments,
            dayStart: {
                hour: this.dayStartHour,
                minute: this.dayStartMinute
            },
            dayEnd: {
                hour: this.dayEndHour,
                minute: this.dayEndMinute
            }
        });
        this.beforeViewRender.emit({
            header: this.days,
            body: this.hours
        });
    }
    /**
     * @return {?}
     */
    refreshAll() {
        this.refreshHeader();
        this.refreshBody();
    }
}
WeekViewComponent.decorators = [
    { type: Component, args: [{
                selector: 'calendar-week-view',
                template: `
    <div class="calendar-week-view">
        <calendar-week-view-header
        [days]="days"
        (eventClicked)="eventClicked.emit({event : $event.event})"
        [allDayEvents]="view?.allDays">
        </calendar-week-view-header>
        <div class="cal-week-container">
            <div class="hour-segments">
                <div class="hour-segment-col" *ngFor="let hour of hours">
                  {{ hour.segments[0].date | date: 'hh a' }}
                </div>
            </div>
            <div class="days-container">
                <div
                *ngFor="let day of view.days"
                [ngClass]="{'isToday': day.isToday }"
                class="day">
                <div *ngIf="day.isToday"
                [style.top.px]="currentTime"
                 class="currentTime"></div>
                <div
                *ngFor="let hour of day.hours" 
                class="day-hour"></div>
                <div
                *ngFor="let event of day.events"
                >
                    <div 
                    *ngIf="!event.event.allDay"
                    [style.top.px]="event.top"
                    [style.height.px]="event.height"
                    (click)="eventClicked.emit({event : event.event})"
                    [style.backgroundColor]="event.event.color ? event.event.color.primary : event.event.calendar.color"
                    class="day-event">
                        {{ event.event.title }}
                    </div>
                </div>
                </div>
            </div>
        </div>
    </div>
  `,
                styles: [`
    .calendar-week-view {
      background-color: white;
    }

    .cal-week-container{
      display: -webkit-box;
      display: -ms-flexbox;
      display: flex;
    }

    .hour-segments{
        height: 1440px;
        display: -webkit-box;
        display: -ms-flexbox;
        display: flex;
        -webkit-box-orient: vertical;
        -webkit-box-direction: normal;
            -ms-flex-direction: column;
                flex-direction: column;
        width: 50px;
        border-right: 1px solid #dddddd;
        border-left: 1px solid #dddddd;
    }
  
    .hour-segment-col:first-child{
      border-top: 1px solid #dddddd;
    }
    .hour-segment-col{
        -webkit-box-flex: 1;
            -ms-flex: 1;
                flex: 1;
        width: 48px;
        border-bottom: 1px solid #dddddd;
        background-color: white;
        font-size: 12px;
        display: -webkit-box;
        display: -ms-flexbox;
        display: flex;
        -webkit-box-align: start;
            -ms-flex-align: start;
                align-items: flex-start;
        -webkit-box-pack: end;
            -ms-flex-pack: end;
                justify-content: flex-end;
        padding-right: 5px;
    }
    .days-container {
      -webkit-box-flex: 1;
          -ms-flex: 1;
              flex: 1;
      display: -webkit-box;
      display: -ms-flexbox;
      display: flex;
      height: 1440px;
    }

    .day-hour {
      -webkit-box-flex: 1;
          -ms-flex: 1;
              flex: 1;
      border-bottom: 1px solid #dddddd;
    }
    .day {
      -webkit-box-flex: 1;
          -ms-flex: 1;
              flex: 1;
      border-right: 1px solid #dddddd;
      height: 1440px;
      position: relative;
      display: -webkit-box;
      display: -ms-flexbox;
      display: flex;
      -webkit-box-orient: vertical;
      -webkit-box-direction: normal;
          -ms-flex-direction: column;
              flex-direction: column; 
    }
    .isToday {
      background-color: #fafafa;
    }
    .day-event{
      position: absolute;
      width: 100%;
      color:white;
      font-size: 12px;
      padding: 5px;
      cursor: pointer;
    }

    .currentTime {
      height: 2px;
      width: 100%;
      background-color: #DB4437;
      position: absolute;
    }
  `]
            },] },
];
/**
 * @nocollapse
 */
WeekViewComponent.ctorParameters = () => [
    { type: ChangeDetectorRef, },
    { type: CalendarUtilsService, },
];
WeekViewComponent.propDecorators = {
    'viewDate': [{ type: Input },],
    'events': [{ type: Input },],
    'weekStartsOn': [{ type: Input },],
    'excludeDays': [{ type: Input },],
    'weekendDays': [{ type: Input },],
    'refresh': [{ type: Input },],
    'precision': [{ type: Input },],
    'hourSegments': [{ type: Input },],
    'hourSegmentHeight': [{ type: Input },],
    'dayStartHour': [{ type: Input },],
    'dayStartMinute': [{ type: Input },],
    'dayEndHour': [{ type: Input },],
    'dayEndMinute': [{ type: Input },],
    'eventWidth': [{ type: Input },],
    'beforeViewRender': [{ type: Output },],
    'eventClicked': [{ type: Output },],
};

class WeekViewHeaderComponent {
    constructor() {
        /**
         * Called when the event title is clicked
         */
        this.eventClicked = new EventEmitter();
        this.dayHeaderClicked = new EventEmitter();
        this.eventDropped = new EventEmitter();
    }
}
WeekViewHeaderComponent.decorators = [
    { type: Component, args: [{
                selector: 'calendar-week-view-header',
                template: `
    <ng-template
    #defaultTemplate
    let-days="days"
    let-locale="locale"
    let-dayHeaderClicked="dayHeaderClicked"
    let-eventDropped="eventDropped">
    <div class="cal-week-day-headers">
    <div class="cal-day-headers">
      <div
        class="cal-header"
        *ngFor="let day of days"
        [class.cal-past]="day.isPast"
        [class.cal-today]="day.isToday"
        [class.cal-future]="day.isFuture"
        [class.cal-weekend]="day.isWeekend"
        [class.cal-drag-over]="day.dragOver"
        [ngClass]="day.cssClass"
        (click)="dayHeaderClicked.emit({day: day})"
        mwlDroppable
        (dragEnter)="day.dragOver = true"
        (dragLeave)="day.dragOver = false"
        (drop)="day.dragOver = false; eventDropped.emit({event: $event.dropData.event, newStart: day.date})">
        <b>{{ day.date | date:'E' }}</b><br>
        <span>{{ day.date | date:'d' }}</span>
      </div>
    </div>
    <div #eventRowContainer class="cal-events-row">
      <div
      *ngFor="let weekEvent of allDayEvents"
      #event
      class="cal-event-container"
      [style.width]="(((100 / days.length) * weekEvent.span)) + '%'"
      [style.marginLeft]="(((100 / days.length) * weekEvent.offset)) + '%'"
      [style.marginTop.px]="0" 
      [style.height.px]="weekEvent.height -6"
      >
      <div
      class="cal-event"
      (click)="eventClicked.emit({event : weekEvent.event})"
      [class.cal-starts-within-week]="weekEvent.startsBeforeWeek"
      [class.cal-ends-within-week]="weekEvent.endsAfterWeek"
      [style.backgroundColor]="weekEvent.event.color ? weekEvent.event.color.primary : weekEvent.event.calendar.color">
        {{ weekEvent.event.title }}
    </div>
    </div>
    </div>
    </div>
    </ng-template>
    <ng-template
    [ngTemplateOutlet]="defaultTemplate"
    [ngTemplateOutletContext]="{days: days, locale: locale, dayHeaderClicked: dayHeaderClicked, eventDropped: eventDropped}">
    </ng-template>
  `,
                styles: [`
    .cal-week-day-headers{
        position: relative;
        margin-left: 50px;
      }
    .cal-day-headers {
          display: -webkit-box;
          display: -ms-flexbox;
          display: flex;
          border: 1px solid #e1e1e1;
          background-color: white;
    }
  
    .cal-day-headers .cal-header {
          -webkit-box-flex: 1;
              -ms-flex: 1;
                  flex: 1;
          text-align: center;
          padding: 5px 0;
          font-size: 12px;
    }

    .cal-events-row{
          border: 1px solid #e1e1e1;
          background-color: white;
          border-top: none;
          position: relative;
    }

    .cal-event {
          padding: 5px;
          font-size: 12px;
          overflow: hidden;
          text-overflow: ellipsis;
          white-space: nowrap;
          height: 100%;
          -webkit-box-sizing: border-box;
                  box-sizing: border-box;
          margin: 2px !important;
          color: white;
          cursor: pointer;
        } 
  `]
            },] },
];
/**
 * @nocollapse
 */
WeekViewHeaderComponent.ctorParameters = () => [];
WeekViewHeaderComponent.propDecorators = {
    'days': [{ type: Input },],
    'allDayEvents': [{ type: Input },],
    'locale': [{ type: Input },],
    'customTemplate': [{ type: Input },],
    'eventClicked': [{ type: Output },],
    'dayHeaderClicked': [{ type: Output },],
    'eventDropped': [{ type: Output },],
};

class WeekModule {
}
WeekModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule
                ],
                declarations: [
                    WeekViewComponent,
                    WeekViewHeaderComponent
                ],
                exports: [
                    WeekViewComponent
                ]
            },] },
];
/**
 * @nocollapse
 */
WeekModule.ctorParameters = () => [];

class ScheduleViewComponent {
    /**
     * @param {?} utils
     */
    constructor(utils) {
        this.utils = utils;
        /**
         * An array of events to display on view.
         */
        this.events = [];
        /**
         * Called when the event title is clicked
         */
        this.eventClicked = new EventEmitter();
    }
    /**
     * @return {?}
     */
    ngOnInit() {
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        if (changes.viewDate ||
            changes.events) {
            this.refreshBody();
        }
    }
    /**
     * @return {?}
     */
    refreshBody() {
        this.view = this.utils.getScheduleView({
            events: this.events,
            viewDate: this.viewDate,
        });
    }
}
ScheduleViewComponent.decorators = [
    { type: Component, args: [{
                selector: 'calendar-schedule-view',
                template: `
    <div class="cal-schedule-view">
      <div class="cal-schedule-day" *ngFor="let day of view">
          <div class="cal-schedule-event-date">
              <div class="cal-schedule-event-date-day">{{ day.date | date:'EEE' }}</div>
              <div class="cal-schedule-event-date-month">{{ day.date | date:'MMM d' }}</div>
          </div>
          <div class="cal-schedule-events">
          <div class="cal-schedule-event" *ngFor="let event of day.events">
              <div 
              [style.backgroundColor]="event.color ? event.color.primary : event.calendar.color"
              class="cal-schedule-event-color mr-2">
              </div>
              <div class="cal-schedule-event-time" *ngIf="!event.allDay">
                  {{ event.start | date:'hh:mm a' }} - {{ event.end | date:'hh:mm a' }}
              </div>
              <div class="cal-schedule-event-time" *ngIf="event.allDay">
                  All day
              </div>
              <div class="cal-schedule-event-title">
                  {{ event.title }}
              </div>
          </div>
          </div>
      </div>
    </div>
  `,
                styles: [`
    .cal-schedule-view {
      border-left: 1px solid #dddddd;
      border-top: 1px solid #dddddd;
      border-right: 1px solid #dddddd; }

    .cal-schedule-day {
      border-bottom: 1px solid #dddddd;
      display: -webkit-box;
      display: -ms-flexbox;
      display: flex;
      padding: 10px; }

    .cal-schedule-event {
      display: -webkit-box;
      display: -ms-flexbox;
      display: flex;
      padding-bottom: 5px;
      -webkit-box-align: center;
          -ms-flex-align: center;
              align-items: center; }

    .cal-schedule-event-date {
      display: -webkit-box;
      display: -ms-flexbox;
      display: flex;
      -webkit-box-orient: vertical;
      -webkit-box-direction: normal;
          -ms-flex-direction: column;
              flex-direction: column;
      font-size: 12px;
      min-width: 50px; }

    .cal-schedule-event-title {
      font-size: 14px;
      font-weight: bold;
      -webkit-box-flex: 1;
          -ms-flex: 1;
              flex: 1; }

    .cal-schedule-event-time {
      font-size: 12px;
      font-weight: normal;
      min-width: 130px;
      -webkit-box-align: center;
          -ms-flex-align: center;
              align-items: center; }

    .cal-schedule-event-color {
      height: 10px !important;
      width: 10px !important;
      border-radius: 50%; }
  `]
            },] },
];
/**
 * @nocollapse
 */
ScheduleViewComponent.ctorParameters = () => [
    { type: CalendarUtilsService, },
];
ScheduleViewComponent.propDecorators = {
    'events': [{ type: Input },],
    'viewDate': [{ type: Input },],
    'refresh': [{ type: Input },],
    'eventClicked': [{ type: Output },],
};

class ScheduleModule {
}
ScheduleModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule
                ],
                declarations: [ScheduleViewComponent],
                exports: [ScheduleViewComponent]
            },] },
];
/**
 * @nocollapse
 */
ScheduleModule.ctorParameters = () => [];

class CalendarHeaderComponent {
    constructor() {
        this.locale = 'fr';
        this.viewChange = new EventEmitter();
        this.viewDateChange = new EventEmitter();
    }
    /**
     * @return {?}
     */
    ngOnInit() {
    }
}
CalendarHeaderComponent.decorators = [
    { type: Component, args: [{
                selector: 'calendar-header',
                template: `

    <div class="row p-2">
      <div class="col-md-3"></div>
        <div class="col-md-2">
          <div class="btn-group">
            <div
              class="btn-circle"
              appCalendarPreviousView
              [view]="view"
              [(viewDate)]="viewDate"
              (viewDateChange)="viewDateChange.next(viewDate)">
              <i class="fa fa-chevron-left"></i>
            </div>
            <div
              class="btn btn-today"
              appCalendarToday
              [(viewDate)]="viewDate"
              (viewDateChange)="viewDateChange.next(viewDate)">
              Today
            </div>
            <div
              class="btn-circle"
              appCalendarNextView
              [view]="view"
              [(viewDate)]="viewDate"
              (viewDateChange)="viewDateChange.next(viewDate)">
              <i class="fa fa-chevron-right"></i>
            </div>
          </div>
        </div>
        <div *ngIf="view === 'week' || view === 'month' || view === 'schedule'" class="col-md-4 todate">
          {{ viewDate | date: 'MMMM yyyy' }}
        </div>
        <div *ngIf="view === 'day'" class="col-md-4 todate">
            {{ viewDate | date: 'EEEE dd MMMM yyyy' }}
        </div>
        <div class="col-md-3">
          <div class="btn-group">
            <div
              class="btn btn-view"
              (click)="viewChange.emit('month')"
              [class.active]="view === 'month'">
              Month
            </div>
            <div
              class="btn btn-view"
              (click)="viewChange.emit('week')"
              [class.active]="view === 'week'">
              Week
            </div>
            <div
              class="btn btn-view"
              (click)="viewChange.emit('day')"
              [class.active]="view === 'day'">
              Day
            </div>
            <div
            class="btn btn-view"
            (click)="viewChange.emit('schedule')"
            [class.active]="view === 'schedule'">
            Schedule
          </div>
          </div>
        </div>
      </div>
  `,
                styles: [`
    .calendar-import-export{
        position: relative;
        display: -webkit-box;
        display: -ms-flexbox;
        display: flex;
    }

    .todate {
        color: #8F8F8F;
        display: -webkit-box;
        display: -ms-flexbox;
        display: flex;
        -webkit-box-pack: center;
            -ms-flex-pack: center;
                justify-content: center;
    }

    .btn {
        border-radius :0; 
        cursor: pointer;
    }
    .btn-today { 
        background-color: #f2f2f2;
        color: #8F8F8F;
    }
    .btn-circle {
        height: 38px;
        width: 38px;
        border-radius: 50%;
        background-color: #ffffff;
        display: -webkit-box;
        display: -ms-flexbox;
        display: flex;
        -webkit-box-align: center;
            -ms-flex-align: center;
                align-items: center;
        -webkit-box-pack: center;
            -ms-flex-pack: center;
                justify-content: center;
        cursor: pointer;
        color: #8F8F8F;
    }

    .btn-circle:hover {
        background-color: #f2f2f2;
    }

    .btn-view {
        color: #8F8F8F;
        background-color: #ffffff;
    }

    .btn-view.active, .btn-view:hover {
        background-color: #f2f2f2;
    }
  `]
            },] },
];
/**
 * @nocollapse
 */
CalendarHeaderComponent.ctorParameters = () => [];
CalendarHeaderComponent.propDecorators = {
    'view': [{ type: Input },],
    'viewDate': [{ type: Input },],
    'locale': [{ type: Input },],
    'events': [{ type: Input },],
    'viewChange': [{ type: Output },],
    'viewDateChange': [{ type: Output },],
};

/**
 * Change the view date to the next view. For example:
 *
 * ```typescript
 * <button
 *  appCalendarNextView
 *  [(viewDate)]="viewDate"
 *  [view]="view">
 *  Next
 * </button>
 * ```
 */
class CalendarNextViewDirective {
    constructor() {
        /**
         * Called when the view date is changed
         */
        this.viewDateChange = new EventEmitter();
    }
    /**
     * @hidden
     * @return {?}
     */
    onClick() {
        const /** @type {?} */ addFn = {
            day: addDays,
            week: addWeeks,
            month: addMonths,
            schedule: addDays
        }[this.view];
        this.viewDateChange.emit(addFn(this.viewDate, 1));
    }
}
CalendarNextViewDirective.decorators = [
    { type: Directive, args: [{
                selector: '[appCalendarNextView]'
            },] },
];
/**
 * @nocollapse
 */
CalendarNextViewDirective.ctorParameters = () => [];
CalendarNextViewDirective.propDecorators = {
    'view': [{ type: Input },],
    'viewDate': [{ type: Input },],
    'viewDateChange': [{ type: Output },],
    'onClick': [{ type: HostListener, args: ['click',] },],
};

/**
 * Change the view date to the previous view. For example:
 *
 * ```typescript
 * <button
 *  appCalendarPreviousView
 *  [(viewDate)]="viewDate"
 *  [view]="view">
 *  Previous
 * </button>
 * ```
 */
class CalendarPreviousViewDirective {
    constructor() {
        /**
         * Called when the view date is changed
         */
        this.viewDateChange = new EventEmitter();
    }
    /**
     * @hidden
     * @return {?}
     */
    onClick() {
        const /** @type {?} */ subFn = {
            day: subDays,
            week: subWeeks,
            month: subMonths,
            schedule: subDays
        }[this.view];
        this.viewDateChange.emit(subFn(this.viewDate, 1));
    }
}
CalendarPreviousViewDirective.decorators = [
    { type: Directive, args: [{
                selector: '[appCalendarPreviousView]'
            },] },
];
/**
 * @nocollapse
 */
CalendarPreviousViewDirective.ctorParameters = () => [];
CalendarPreviousViewDirective.propDecorators = {
    'view': [{ type: Input },],
    'viewDate': [{ type: Input },],
    'viewDateChange': [{ type: Output },],
    'onClick': [{ type: HostListener, args: ['click',] },],
};

/**
 * Change the view date to the current day. For example:
 *
 * ```typescript
 * <button
 *  appCalendarToday
 *  [(viewDate)]="viewDate">
 *  Today
 * </button>
 * ```
 */
class CalendarTodayDirective {
    constructor() {
        /**
         * Called when the view date is changed
         */
        this.viewDateChange = new EventEmitter();
    }
    /**
     * @hidden
     * @return {?}
     */
    onClick() {
        this.viewDateChange.emit(startOfToday());
    }
}
CalendarTodayDirective.decorators = [
    { type: Directive, args: [{
                selector: '[appCalendarToday]'
            },] },
];
/**
 * @nocollapse
 */
CalendarTodayDirective.ctorParameters = () => [];
CalendarTodayDirective.propDecorators = {
    'viewDate': [{ type: Input },],
    'viewDateChange': [{ type: Output },],
    'onClick': [{ type: HostListener, args: ['click',] },],
};

class HeaderModule {
}
HeaderModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule
                ],
                declarations: [
                    CalendarHeaderComponent,
                    CalendarNextViewDirective,
                    CalendarPreviousViewDirective,
                    CalendarTodayDirective
                ],
                exports: [
                    CalendarHeaderComponent,
                    CalendarNextViewDirective,
                    CalendarPreviousViewDirective,
                    CalendarTodayDirective
                ]
            },] },
];
/**
 * @nocollapse
 */
HeaderModule.ctorParameters = () => [];

class CalendarModule {
    /**
     * @param {?=} config
     * @return {?}
     */
    static forRoot(config = {}) {
        return {
            ngModule: CalendarModule,
            providers: [
                DraggableHelper,
                CalendarUtilsService
            ]
        };
    }
}
CalendarModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                    FormsModule,
                    ReactiveFormsModule,
                    DayModule,
                    MonthModule,
                    WeekModule,
                    ScheduleModule,
                    HeaderModule,
                ],
                exports: [
                    DayModule,
                    MonthModule,
                    WeekModule,
                    ScheduleModule,
                    HeaderModule,
                ],
                declarations: []
            },] },
];
/**
 * @nocollapse
 */
CalendarModule.ctorParameters = () => [];

/**
 * Generated bundle index. Do not edit.
 */

export { CalendarModule, DayModule, MonthModule, WeekModule, ScheduleModule, EventActionComponent as ɵf, EventTitleComponent as ɵg, DayViewAllDayEventComponent as ɵc, DayViewEventComponent as ɵd, DayViewHourSegmentComponent as ɵe, DayViewComponent as ɵa, CalendarNextViewDirective as ɵq, CalendarPreviousViewDirective as ɵr, CalendarTodayDirective as ɵs, CalendarHeaderComponent as ɵp, HeaderModule as ɵo, MonthCellComponent as ɵi, MonthViewHeaderComponent as ɵj, MonthViewComponent as ɵh, OpenDayEventsComponent as ɵk, ScheduleViewComponent as ɵn, CalendarUtilsService as ɵb, WeekViewHeaderComponent as ɵm, WeekViewComponent as ɵl };
//# sourceMappingURL=ngx-calendar.js.map
